@@ -10,13 +10,16 @@ See https://cadet-process.readthedocs.io for complete documentation.
 """
 # Version information
 name = 'CADET-Process'
-__version__ = '0.6.3'
+__version__ = '0.7.0'
 
 # Imports
 from .CADETProcessError import *
 
-from . import settings
 from . import log
+
+from .settings import Settings
+settings = Settings()
+
 from . import dataStructure
 from . import transform
 from . import plotting
@@ -35,4 +38,4 @@ from . import simulator
 from . import fractionation
 from . import equilibria
 from . import modelBuilder
-
+from . import tools
