@@ -10,6 +10,7 @@ from CADETProcess.dataStructure import String
 from CADETProcess.dynamicEvents import EventHandler
 from CADETProcess import plotting
 from CADETProcess.performance import Performance
+from CADETProcess.solution import slice_solution
 from CADETProcess import SimulationResults
 
 from CADETProcess.fractionation.fractions import Fraction, FractionPool
@@ -32,8 +33,17 @@ class Fractionator(EventHandler):
         'productivity', 'eluent_consumption'
     ]
 
-    def __init__(self, simulation_results, *args, **kwargs):
+    def __init__(
+            self,
+            simulation_results,
+            components=None,
+            use_total_concentration_components=True,
+            *args, **kwargs):
+
+        self.components = components
+        self.use_total_concentration_components = use_total_concentration_components
         self.simulation_results = simulation_results
+        self._cycle_time = None
 
         super().__init__(*args, **kwargs)
 
@@ -53,6 +63,29 @@ class Fractionator(EventHandler):
 
         self._simulation_results = simulation_results
 
+        self._chromatograms = [
+            slice_solution(
+                chrom,
+                components=self.components,
+                use_total_concentration_components=self.use_total_concentration_components
+            )
+            for chrom in simulation_results.chromatograms
+        ]
+
+        m_feed = np.zeros((self.component_system.n_comp, ))
+        counter = 0
+        for comp, indices in simulation_results.component_system.indices.items():
+            if comp in self.component_system.names:
+                m_feed_comp = simulation_results.process.m_feed[indices]
+                if self.use_total_concentration_components:
+                    m_feed[counter] = np.sum(m_feed_comp)
+                    counter += 1
+                else:
+                    n_species = len(indices)
+                    m_feed[counter:counter+n_species] = m_feed_comp
+                    counter += n_species
+        self.m_feed = m_feed
+
         self._fractionation_states = Dict({
             chrom: []
             for chrom in self.chromatograms
@@ -86,13 +119,13 @@ class Fractionator(EventHandler):
         """list: Chromatograms to be fractionized.
 
         See Also
-        ---------
+        --------
         add_chromatogram
         SoltionIO
         reset
         cycle_time
         """
-        return self.simulation_results.chromatograms
+        return self._chromatograms
 
     @property
     def chromatograms_dict(self):
@@ -130,7 +163,13 @@ class Fractionator(EventHandler):
     @property
     def cycle_time(self):
         """float: cycle time"""
-        return self.process.cycle_time
+        if self._cycle_time is None:
+            return self.process.cycle_time
+        return self._cycle_time
+
+    @cycle_time.setter
+    def cycle_time(self, cycle_time):
+        self._cycle_time = cycle_time
 
     @property
     def time(self):
@@ -138,7 +177,8 @@ class Fractionator(EventHandler):
         return self.chromatograms[0].time
 
     @plotting.create_and_save_figure
-    def plot_fraction_signal(self, chromatogram=None, ax=None, *args, **kwargs):
+    def plot_fraction_signal(
+            self, chromatogram=None, ax=None, *args, **kwargs):
         """Plot the signal without the waste fractions.
 
         Parameters
@@ -160,8 +200,9 @@ class Fractionator(EventHandler):
 
         """
         if chromatogram is None:
-            chromatogram = self.chromatograms[0]
-        elif isinstance(chromatogram, str):
+            chromatogram = \
+                list(self.performer_timelines['fractionation_states'].keys())[0]
+        if isinstance(chromatogram, str):
             chromatogram = self.chromatograms_dict[chromatogram]
 
         time_line = \
@@ -176,7 +217,7 @@ class Fractionator(EventHandler):
         except KeyError:
             end = np.max(chromatogram.time)
 
-        ax = chromatogram.plot(show=False, ax=ax, *args, **kwargs)
+        _,  ax = chromatogram.plot(show=False, ax=ax, *args, **kwargs)
 
         y_max = 1.1*np.max(chromatogram.solution)
 
@@ -270,6 +311,11 @@ class Fractionator(EventHandler):
 
         self._fractionation_states[chrom] = fractionation_state
 
+    @property
+    def n_fractions_per_pool(self):
+        """list: number of fractions per pool."""
+        return [pool.n_fractions for pool in self.fraction_pools]
+
     @property
     def fraction_pools(self):
         """List of the component and waste fraction pools.
@@ -384,7 +430,7 @@ class Fractionator(EventHandler):
     def recovery(self):
         """ndarray: Component recovery yield in corresponding fraction pool."""
         with np.errstate(divide='ignore', invalid='ignore'):
-            recovery = self.mass / self.process.m_feed
+            recovery = self.mass / self.m_feed
 
         return np.nan_to_num(recovery)
 
@@ -462,7 +508,7 @@ class Fractionator(EventHandler):
             purity_required = [purity_required]*self.n_comp
         elif len(purity_required) != self.n_comp:
             raise ValueError(
-                f'Expected array with size {self.chromatogram.n_comp}'
+                f'Expected purity array with size {self.n_comp}'
             )
 
         self._events = []
