@@ -15,6 +15,9 @@ from CADETProcess.processModel import TubularReactorBase, Cstr
 from CADETProcess.solution import SolutionBase
 
 
+__all__ = ['CarouselBuilder', 'SerialZone', 'ParallelZone']
+
+
 class CarouselBuilder(metaclass=StructMeta):
     switch_time = UnsignedFloat()
 
@@ -131,8 +134,6 @@ class CarouselBuilder(metaclass=StructMeta):
             flow_sheet.set_output_state(zone.outlet_unit, output_state)
 
             zone_flow_flow_rate = flow_rates[zone.name].total_out
-            flow_sheet[zone.inlet_unit.name].flow_rate = zone_flow_flow_rate
-            flow_sheet[zone.outlet_unit.name].flow_rate = zone_flow_flow_rate
 
     def add_intra_zone_connections(self, flow_sheet):
         """Add connections within zones."""
@@ -286,7 +287,7 @@ class CarouselBuilder(metaclass=StructMeta):
 class ZoneBaseClass(UnitBaseClass):
     n_columns = UnsignedInteger()
     flow_direction = Integer(default=1)
-    _valve_dead_volume = 1e-6
+    _valve_dead_volume = 1e-12
 
     def __init__(
             self,
