@@ -1,4 +1,3 @@
-import time
 import warnings
 
 from scipy import optimize
@@ -9,7 +8,7 @@ from CADETProcess import CADETProcessError
 from CADETProcess.dataStructure import (
     Bool, Switch, UnsignedInteger, UnsignedFloat
 )
-from CADETProcess.optimization import OptimizerBase, OptimizationResults
+from CADETProcess.optimization import OptimizerBase
 
 
 class SciPyInterface(OptimizerBase):
@@ -23,7 +22,7 @@ class SciPyInterface(OptimizerBase):
     tol = UnsignedFloat()
     jac = '2-point'
 
-    def run(self, optimization_problem):
+    def run(self, optimization_problem, x0=None):
         """Solve the optimization problem using any of the scipy methods.
 
         Returns
@@ -31,6 +30,8 @@ class SciPyInterface(OptimizerBase):
         results : OptimizationResults
             Optimization results including optimization_problem and solver
             configuration.
+        x0 : list, optional
+            Initial values.
 
         See Also
         --------
@@ -43,44 +44,56 @@ class SciPyInterface(OptimizerBase):
         scipy.optimize.minimize
 
         """
+        self.n_evals = 0
+
         if optimization_problem.n_objectives > 1:
             raise CADETProcessError("Can only handle single objective.")
 
         def objective_function(x):
-            return optimization_problem.evaluate_objectives(x, cache=self.progress.cache)[0]
+            return optimization_problem.evaluate_objectives(x)[0]
+
+        def callback_function(x, state=None):
+            """Internal callback to report progress after evaluation.
+
+            Note
+            ----
+            Currently, this evaluates all functions again. This should not be a problem
+            since objectives and constraints are automatically cached.
+
+            Unfortunately, only `trust-constr` returns a `state` which contains the
+            current best point. Hence, the internal pareto front is used.
+            """
+
+            self.n_evals += 1
+
+            x = x.tolist()
+            f = optimization_problem.evaluate_objectives(x)
+            g = optimization_problem.evaluate_nonlinear_constraints(x)
+            cv = optimization_problem.evaluate_nonlinear_constraints_violation(x)
+
+            self.run_post_evaluation_processing(x, f, g, cv, self.n_evals)
+
+            return False
+
+        if x0 is None:
+            x0 = optimization_problem.create_initial_values(1, method='chebyshev')
 
-        start = time.time()
         with warnings.catch_warnings():
             warnings.filterwarnings('ignore', category=OptimizeWarning)
             warnings.filterwarnings('ignore', category=RuntimeWarning)
             scipy_results = optimize.minimize(
                 objective_function,
-                x0=optimization_problem.x0,
+                x0=x0,
                 method=str(self),
                 tol=self.tol,
                 jac=self.jac,
                 constraints=self.get_constraint_objects(optimization_problem),
-                options=self.options
-                )
-        elapsed = time.time() - start
+                options=self.options,
+                callback=callback_function,
+            )
 
-        x = list(scipy_results.x)
-        f = optimization_problem.evaluate_objectives(x)
-        g = optimization_problem.evaluate_nonlinear_constraints(x)
-
-        results = OptimizationResults(
-            optimization_problem=optimization_problem,
-            optimizer=str(self),
-            optimizer_options=self.options,
-            exit_flag=scipy_results.status,
-            exit_message=scipy_results.message,
-            time_elapsed=elapsed,
-            x=x,
-            f=f,
-            g=g,
-        )
-
-        return results
+        self.results.exit_flag = scipy_results.status
+        self.results.exit_message = scipy_results.message
 
     def get_bounds(self, optimization_problem):
         """Returns the optimized bounds of a given optimization_problem as a
@@ -95,8 +108,8 @@ class SciPyInterface(OptimizerBase):
             Returns the optimized bounds as an object called bounds.
         """
         return optimize.Bounds(
-            optimization_problem.lower_bounds,
-            optimization_problem.upper_bounds,
+            optimization_problem.lower_bounds_independent_transformed,
+            optimization_problem.upper_bounds_independent_transformed,
             keep_feasible=True
         )
 
@@ -113,7 +126,7 @@ class SciPyInterface(OptimizerBase):
             List containing  a sorted list of all constraints of an
             optimization_problem, if they're not None.
 
-        See also
+        See Also
         --------
         lincon_obj
         lincon_obj
@@ -139,12 +152,15 @@ class SciPyInterface(OptimizerBase):
             Linear Constraint object with optimized upper and lower bounds of b
             of the optimization_problem.
 
-        See also
+        See Also
         --------
         constraint_objects
         A
         b
         """
+        if optimization_problem.n_linear_constraints == 0:
+            return None
+
         lb = [-np.inf]*len(optimization_problem.b)
         ub = optimization_problem.b
 
@@ -168,13 +184,13 @@ class SciPyInterface(OptimizerBase):
             Linear equality Constraint object with optimized upper and lower
             bounds of beq of the optimization_problem.
 
-        See also
+        See Also
         --------
         constraint_objects
         Aeq
         beq
         """
-        if len(optimization_problem.beq) == 0:
+        if optimization_problem.n_linear_equality_constraints == 0:
             return None
 
         lb = optimization_problem.beq
@@ -209,19 +225,20 @@ class SciPyInterface(OptimizerBase):
             Linear equality Constraint object with optimized upper and lower
             bounds of beq of the optimization_problem.
 
-        See also
+        See Also
         --------
         constraint_objects
         nonlinear_constraints
         """
+        if optimization_problem.n_nonlinear_constraints == 0:
+            return [None]
+
         opt = optimization_problem
-        if opt.nonlinear_constraints is None:
-            return None
 
         def makeConstraint(i):
             constr = optimize.NonlinearConstraint(
-                lambda x: opt.evaluate_nonlinear_constraints(x, cache=self.progress.cache)[i],
-                lb=-np.inf, ub=0,
+                lambda x: opt.evaluate_nonlinear_constraints(x)[i],
+                lb=-np.inf, ub=opt.nonlinear_constraints_bounds[i],
                 finite_diff_rel_step=self.finite_diff_rel_step,
                 keep_feasible=True
                 )
@@ -247,18 +264,23 @@ class TrustConstr(SciPyInterface):
     running the optimization.
 
     """
+    supports_linear_constraints = True
+    supports_linear_equality_constraints = True
+    supports_nonlinear_constraints = True
+
     gtol = UnsignedFloat(default=1e-6)
+    cv_tol = gtol
     xtol = UnsignedFloat(default=1e-8)
     barrier_tol = UnsignedFloat(default=1e-8)
     initial_constr_penalty = UnsignedFloat(default=1.0)
     initial_tr_radius = UnsignedFloat(default=1.0)
-    initial_barrier_parameter = UnsignedFloat(default=0.01)
-    initial_barrier_tolerance = UnsignedFloat(default=0.01)
+    initial_barrier_parameter = UnsignedFloat(default=0.1)
+    initial_barrier_tolerance = UnsignedFloat(default=0.1)
     factorization_method = None
     maxiter = UnsignedInteger(default=1000)
     verbose = UnsignedInteger(default=0)
     disp = Bool(default=False)
-    _options = [
+    _options = OptimizerBase._options + [
         'gtol', 'xtol', 'barrier_tol', 'finite_diff_rel_step',
         'initial_constr_penalty',
         'initial_tr_radius', 'initial_barrier_parameter',
@@ -282,11 +304,16 @@ class COBYLA(SciPyInterface):
     running the optimization.
 
     """
+    supports_linear_constraints = True
+    supports_linear_equality_constraints = True
+    supports_nonlinear_constraints = True
+
     rhobeg = UnsignedFloat(default=1)
     maxiter = UnsignedInteger(default=10000)
     disp = Bool(default=False)
     catol = UnsignedFloat(default=0.0002)
-    _options = ['rhobeg', 'maxiter', 'disp', 'catol']
+    cv_tol = catol
+    _options = OptimizerBase._options + ['rhobeg', 'maxiter', 'disp', 'catol']
 
 
 class NelderMead(SciPyInterface):
@@ -299,10 +326,13 @@ class NelderMead(SciPyInterface):
     xatol = UnsignedFloat(default=0.01)
     fatol = UnsignedFloat(default=0.01)
     adaptive = Bool(default=True)
-    _options = [
+    _options = OptimizerBase._options + [
         'maxiter', 'maxfev', 'initial_simplex', 'xatol', 'fatol', 'adaptive'
     ]
 
+    def __str__(self):
+        return 'Nelder-Mead'
+
 
 class SLSQP(SciPyInterface):
     """Class from scipy for optimization with SLSQP as method for the
@@ -313,7 +343,11 @@ class SLSQP(SciPyInterface):
     variable options as a dictionary and implements the abstract method run for
     running the optimization.
     """
+    supports_linear_constraints = True
+    supports_linear_equality_constraints = True
+    supports_nonlinear_constraints = True
+
     ftol = UnsignedFloat(default=1e-2)
     eps = UnsignedFloat(default=1e-6)
     disp = Bool(default=False)
-    _options = ['ftol', 'eps', 'disp']
+    _options = OptimizerBase._options + ['ftol', 'eps', 'disp']
