@@ -1,24 +1,90 @@
+"""
+=============================================
+Performance (:mod:`CADETProcess.performance`)
+=============================================
+
+.. currentmodule:: CADETProcess.performance
+
+
+Performance data
+================
+
+Classes for storing the performance parameters after fractionation.
+
+.. autosummary::
+    :toctree: generated/
+
+    Performance
+    RankedPerformance
+
+Performance Indicators
+======================
+
+Individual performance indicators (extracted from Performance).
+Mostly convenince method.
+
+.. autosummary::
+    :toctree: generated/
+
+    PerformanceIndicator
+    Mass
+    Recovery
+    Productivity
+    EluentConsumption
+    Purity
+    Concentration
+    PerformanceProduct
+
+Note
+----
+Performance Indicators might be deprecated in future since with new evaluation chains
+it's no longer required for setting up as optimization problem.
+
+"""
+
 import numpy as np
 
 from CADETProcess import CADETProcessError
-
 from CADETProcess.dataStructure import Structure
-from CADETProcess.dataStructure import NdArray, DependentlySizedNdArray
 from CADETProcess.metric import MetricBase
-
+from CADETProcess.dataStructure import DependentlySizedNdArray
 from CADETProcess.processModel import ComponentSystem
 
 
 class Performance(Structure):
     """Class for storing the performance parameters after fractionation.
 
+    Attributes
+    ----------
+    mass : np.ndarray
+        Mass of each component in the system after fractionation.
+        Size depends on number of components.
+    concentration : np.ndarray
+        Concentration of each component in the system after fractionation.
+        Size depends on number of components.
+    purity : np.ndarray
+        Purity of each component in the system after fractionation.
+        Size depends on number of components.
+    recovery : np.ndarray
+        Recovery of each component in the system after fractionation.
+        Size depends on number of components.
+    productivity : np.ndarray
+        Productivity of each component in the system after fractionation.
+        Size depends on number of components.
+    eluent_consumption : np.ndarray
+        Eluent consumption of each component in the system after fractionation.
+        Size depends on number of components.
+    component_system : ComponentSystem
+        The component system used for fractionation.
+        If not provided, a default component system is used.
+
     See Also
     --------
-    Fractionation
-    ProcessMeta
+    CADETProcess.fractionation
     RankedPerformance
 
     """
+
     _performance_keys = [
         'mass', 'concentration', 'purity', 'recovery',
         'productivity', 'eluent_consumption'
@@ -34,6 +100,25 @@ class Performance(Structure):
     def __init__(
             self, mass, concentration, purity, recovery,
             productivity, eluent_consumption, component_system=None):
+        """Initialize Performance.
+
+        Parameters
+        ----------
+        mass : ndarray
+            The mass of each component.
+        concentration : ndarray
+            The concentration of each component.
+        purity : ndarray
+            The purity of each component.
+        recovery : ndarray
+            The recovery of each component.
+        productivity : ndarray
+            The productivity of each component.
+        eluent_consumption : ndarray
+            The eluent consumption of each component.
+        component_system : ComponentSystem
+            The ComponentSystem object that describes the system's components.
+        """
 
         if component_system is None:
             component_system = ComponentSystem(mass.shape[0])
@@ -48,19 +133,23 @@ class Performance(Structure):
 
     @property
     def n_comp(self):
+        """int: Number of components in the system."""
         return self.component_system.n_comp
 
     def to_dict(self):
+        """Return a dictionary representation of the object."""
         return {key: getattr(self, key).tolist()
                 for key in self._performance_keys}
 
     def __getitem__(self, item):
+        """Get an attribute of the object by its name."""
         if item not in self._performance_keys:
             raise AttributeError('Not a valid performance parameter')
 
         return getattr(self, item)
 
     def __repr__(self):
+        """String representation of the object."""
         return \
             f'{self.__class__.__name__}(mass={np.array_repr(self.mass)}, '\
             f'concentration={np.array_repr(self.concentration)}, '\
@@ -100,7 +189,7 @@ class RankedPerformance():
 
     @ranking.setter
     def ranking(self, ranking):
-        n_metrics = self.component_system.n_comp - self.n_exclude
+        n_metrics = self.component_system.n_comp
 
         if isinstance(ranking, (float, int)):
             ranking = n_metrics * [ranking]
@@ -138,21 +227,23 @@ class RankedPerformance():
 
 
 class PerformanceIndicator(MetricBase):
-    def __init__(self, exclude=None, ranking=None):
-        self.exclude = exclude
+    """Base class for performance indicators used in optimization and fractionation.
+
+    See Also
+    --------
+    RankedPerformance
+    """
+
+    def __init__(self, ranking=None):
+        """Initialize PerformanceIndicator.
+
+        Parameters
+        ----------
+        ranking : list of floats, optional
+            Weights to rank individual compoments. If None, all compoments are ranke
+        """
         self.ranking = ranking
 
-    @property
-    def exclude(self):
-        return self._exclude
-
-    @exclude.setter
-    def exclude(self, exclude=None):
-        if exclude is None:
-            exclude = []
-
-        self._exclude = exclude
-
     @property
     def ranking(self):
         return self._ranking
@@ -166,6 +257,18 @@ class PerformanceIndicator(MetricBase):
         return 0
 
     def evaluate(self, performance):
+        """Evaluate the performance indicator for the given performance data.
+
+        Parameters
+        ----------
+        performance : Performance
+            Object containing performance data.
+
+        Returns
+        -------
+        list
+            List of performance indicator values.
+        """
         try:
             performance = performance.performance
         except AttributeError:
@@ -181,46 +284,107 @@ class PerformanceIndicator(MetricBase):
         else:
             metric = []
             for i, comp in enumerate(performance.component_system):
-                if comp.name in self.exclude:
-                    continue
                 metric.append(value[i])
 
         return metric
 
     __call__ = evaluate
 
+    def __str__(self):
+        """String representation of the class."""
+        return self.__class__.__name__
+
 
 class Mass(PerformanceIndicator):
+    """Performance indicator based on the mass of each component in the system.
+
+    See Also
+    --------
+    PerformanceIndicator
+
+    """
+
     def _evaluate(self, performance):
         return - performance.mass
 
 
 class Recovery(PerformanceIndicator):
+    """Performance indicator based on the recovery of each component in the system.
+
+    See Also
+    --------
+    PerformanceIndicator
+
+    """
+
     def _evaluate(self, performance):
         return - performance.recovery
 
 
 class Productivity(PerformanceIndicator):
+    """Performance indicator based on the productivity of each component in the system.
+
+    See Also
+    --------
+    PerformanceIndicator
+
+    """
+
     def _evaluate(self, performance):
         return - performance.productivity
 
 
 class EluentConsumption(PerformanceIndicator):
+    """Performance indicator based on the specific eluent consumption of each component.
+
+    See Also
+    --------
+    PerformanceIndicator
+
+    """
+
     def _evaluate(self, performance):
         return - performance.eluent_consumption
 
 
 class Purity(PerformanceIndicator):
+    """Performance indicator based on the purity of each component in the system.
+
+    See Also
+    --------
+    PerformanceIndicator
+
+    """
+
     def _evaluate(self, performance):
         return - performance.purity
 
 
 class Concentration(PerformanceIndicator):
+    """Performance indicator based on the concentration of each component in the system.
+
+    See Also
+    --------
+    PerformanceIndicator
+
+    """
+
     def _evaluate(self, performance):
         return - performance.concentration
 
 
 class PerformanceProduct(PerformanceIndicator):
+    """Performance indicator based on the product of several performance indicators.
+
+    See Also
+    --------
+    Productivity
+    Recovery
+    EluentConsumption
+    PerformanceIndicator
+
+    """
+
     def _evaluate(self, performance):
         return \
             - performance.productivity \
