@@ -1,12 +1,38 @@
+from warnings import warn
+
+import numpy as np
+
 from CADETProcess import CADETProcessError
 
 from CADETProcess.dataStructure import frozen_attributes
 from CADETProcess.dataStructure import StructMeta
-from CADETProcess.dataStructure import Bool, String, Integer, \
-    UnsignedInteger, UnsignedFloat, DependentlySizedUnsignedList
+from CADETProcess.dataStructure import Bool, String, \
+    RangedInteger, UnsignedInteger, UnsignedFloat, DependentlySizedList, \
+    DependentlySizedRangedIntegerList, DependentlySizedUnsignedIntegerList, \
+    DependentlySizedUnsignedList, DependentlyModulatedUnsignedList
 
 from .componentSystem import ComponentSystem
 
+__all__ = [
+    'BindingBaseClass',
+    'NoBinding',
+    'Linear',
+    'Langmuir',
+    'LangmuirLDF',
+    'BiLangmuir',
+    'BiLangmuirLDF',
+    'FreundlichLDF',
+    'StericMassAction',
+    'AntiLangmuir',
+    'Spreading',
+    'SelfAssociation',
+    'BiStericMassAction',
+    'MultistateStericMassAction',
+    'SimplifiedMultistateStericMassAction',
+    'Saska',
+    'GeneralizedIonExchange',
+]
+
 
 @frozen_attributes
 class BindingBaseClass(metaclass=StructMeta):
@@ -20,6 +46,14 @@ class BindingBaseClass(metaclass=StructMeta):
         system of components.
     n_comp : int
         number of components.
+    n_binding_sites : int
+        Number of binding sites.
+        Relevant for Multi-Site isotherms such as Bi-Langmuir.
+        The default is 1.
+    bound_states : list of unsigned integers.
+        Number of binding sites per component.
+    non_binding_component_indices : list
+        (Hardcoded) list of non binding modifier components (e.g. pH).
     is_kinetic : bool
         If False, adsorption is assumed to be in rapid equilibriu.
         The default is True.
@@ -27,11 +61,18 @@ class BindingBaseClass(metaclass=StructMeta):
         dict with parameter values.
 
     """
+
     name = String()
     is_kinetic = Bool(default=True)
-    n_states = Integer(lb=1, ub=1, default=1)
+
+    n_binding_sites = RangedInteger(lb=1, ub=1, default=1)
+    _bound_states = DependentlySizedRangedIntegerList(
+        dep=('n_binding_sites', 'n_comp'), lb=0, ub=1, default=1
+    )
+    non_binding_component_indices = []
 
     _parameter_names = ['is_kinetic']
+    _required_parameters = []
 
     def __init__(self, component_system, name=None):
         self.component_system = component_system
@@ -60,6 +101,27 @@ class BindingBaseClass(metaclass=StructMeta):
     def n_comp(self):
         return self.component_system.n_comp
 
+    @property
+    def bound_states(self):
+        bound_states = self._bound_states
+        for i in self.non_binding_component_indices:
+            bound_states[i] = 0
+        return bound_states
+
+    @bound_states.setter
+    def bound_states(self, bound_states):
+        indices = self.non_binding_component_indices
+        if any(bound_states[i] > 0 for i in indices):
+            raise CADETProcessError(
+                "Cannot set bound state for non-binding component."
+            )
+
+        self._bound_states = bound_states
+
+    @property
+    def n_bound_states(self):
+        return sum(self.bound_states)
+
     @property
     def parameters(self):
         """dict: Dictionary with parameter values."""
@@ -72,6 +134,41 @@ class BindingBaseClass(metaclass=StructMeta):
                 raise CADETProcessError('Not a valid parameter')
             setattr(self, param, value)
 
+    @property
+    def required_parameters(self):
+        return self._required_parameters
+
+    @property
+    def missing_parameters(self):
+        """list: List of missing parameters."""
+        missing_parameters = []
+        for param in self.required_parameters:
+            if getattr(self, param) is None:
+                missing_parameters.append(param)
+
+        return missing_parameters
+
+    def check_required_parameters(self):
+        """Check if al required parameters are set.
+
+        Returns
+        -------
+        bool
+            True if all parameters are set. False otherwise.
+
+        Raises
+        ------
+        Warning
+            If any required parameters are missing.
+
+        """
+        if len(self.missing_parameters) == 0:
+            return True
+        else:
+            for param in self.missing_parameters:
+                warn(f'Binding Model {self.name}: Missing parameter "{param}".')
+            return False
+
     def __repr__(self):
         return f"{self.__class__.__name__}(\
             component_system={self.component_system}, name={self.name})')"
@@ -104,8 +201,9 @@ class Linear(BindingBaseClass):
         Desorption rate constants.
 
     """
-    adsorption_rate = DependentlySizedUnsignedList(dep='n_comp', default=0)
-    desorption_rate = DependentlySizedUnsignedList(dep='n_comp', default=1)
+
+    adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
+    desorption_rate = DependentlySizedUnsignedList(dep='n_comp')
 
     _parameter_names = BindingBaseClass._parameter_names + [
         'adsorption_rate', 'desorption_rate'
@@ -125,13 +223,17 @@ class Langmuir(BindingBaseClass):
         Maximum adsorption capacities.
 
     """
+
     adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
-    desorption_rate = DependentlySizedUnsignedList(dep='n_comp', default=1)
+    desorption_rate = DependentlySizedUnsignedList(dep='n_comp')
     capacity = DependentlySizedUnsignedList(dep='n_comp')
 
     _parameter_names = BindingBaseClass._parameter_names + [
         'adsorption_rate', 'desorption_rate', 'capacity'
     ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'adsorption_rate', 'desorption_rate', 'capacity'
+    ]
 
 
 class LangmuirLDF(BindingBaseClass):
@@ -147,11 +249,12 @@ class LangmuirLDF(BindingBaseClass):
         Maximum adsorption capacities.
 
     """
+
     equilibrium_constant = DependentlySizedUnsignedList(
         dep=('n_comp', 'n_states')
     )
     driving_force_coefficient = DependentlySizedUnsignedList(
-        dep=('n_comp', 'n_states'), default=1
+        dep=('n_comp', 'n_states')
     )
     capacity = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'))
 
@@ -160,6 +263,11 @@ class LangmuirLDF(BindingBaseClass):
         'driving_force_coefficient',
         'capacity',
     ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'equilibrium_constant',
+        'driving_force_coefficient',
+        'capacity',
+    ]
 
 
 class BiLangmuir(BindingBaseClass):
@@ -175,29 +283,29 @@ class BiLangmuir(BindingBaseClass):
         Maximum adsorption capacities.
 
     """
-    adsorption_rate = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'))
-    desorption_rate = DependentlySizedUnsignedList(
-        dep=('n_comp', 'n_states'), default=1
-    )
-    capacity = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'))
-    n_states = UnsignedInteger()
+
+    n_binding_sites = UnsignedInteger(default=2)
+
+    adsorption_rate = DependentlySizedUnsignedList(dep='n_bound_states')
+    desorption_rate = DependentlySizedUnsignedList(dep='n_bound_states')
+    capacity = DependentlySizedUnsignedList(dep='n_bound_states')
 
     _parameter_names = BindingBaseClass._parameter_names + [
         'adsorption_rate',
         'desorption_rate',
         'capacity',
-        'n_states'
+    ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'adsorption_rate',
+        'desorption_rate',
+        'capacity',
     ]
 
-    def __init__(self, *args, n_states=2, **kwargs):
-        self.n_states = n_states
+    def __init__(self, *args, n_binding_sites=2, **kwargs):
+        self.n_binding_sites = n_binding_sites
 
         super().__init__(*args, **kwargs)
 
-    @property
-    def n_total_states(self):
-        return self.n_comp * self.n_states
-
 
 class BiLangmuirLDF(BindingBaseClass):
     """Parameters for Multi Component Bi-Langmuir binding model.
@@ -212,27 +320,29 @@ class BiLangmuirLDF(BindingBaseClass):
         Maximum adsorption capacities.
 
     """
-    equilibrium_constant = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'))
-    driving_force_coefficient = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'), default=1)
-    capacity = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'))
-    n_states = UnsignedInteger()
+
+    n_binding_sites = UnsignedInteger(default=2)
+
+    equilibrium_constant = DependentlySizedUnsignedList(dep='n_bound_states')
+    driving_force_coefficient = DependentlySizedUnsignedList(dep='n_bound_states')
+    capacity = DependentlySizedUnsignedList(dep='n_bound_states')
 
     _parameter_names = BindingBaseClass._parameter_names + [
         'equilibrium_constant',
         'driving_force_coefficient',
         'capacity',
-        'n_states'
+    ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'equilibrium_constant',
+        'driving_force_coefficient',
+        'capacity',
     ]
 
-    def __init__(self, *args, n_states=2, **kwargs):
-        self.n_states = n_states
+    def __init__(self, *args, n_binding_sites=2, **kwargs):
+        self.n_binding_sites = n_binding_sites
 
         super().__init__(*args, **kwargs)
 
-    @property
-    def n_total_states(self):
-        return self.n_comp * self.n_states
-
 
 class FreundlichLDF(BindingBaseClass):
     """Parameters for the Freundlich isotherm model.
@@ -247,6 +357,7 @@ class FreundlichLDF(BindingBaseClass):
         Exponent for each component. Length depends on n_comp.
 
     """
+
     driving_force_coefficient = DependentlySizedUnsignedList(dep='n_comp')
     freundlich_coefficient = DependentlySizedUnsignedList(dep='n_comp')
     exponent = DependentlySizedUnsignedList(dep='n_comp')
@@ -256,6 +367,11 @@ class FreundlichLDF(BindingBaseClass):
         'freundlich_coefficient',
         'exponent',
     ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'driving_force_coefficient',
+        'freundlich_coefficient',
+        'exponent',
+    ]
 
 
 class StericMassAction(BindingBaseClass):
@@ -278,28 +394,66 @@ class StericMassAction(BindingBaseClass):
         Stationary phase capacity (monovalent salt counterions); The total
         number of binding sites available on the resin surface.
     reference_liquid_phase_conc : unsigned float.
-        Reference liquid phase concentration (optional, default value = 1.0).
+        Reference liquid phase concentration.
+        The default is 1.0
     reference_solid_phase_conc : unsigned float.
-        Reference liquid phase concentration (optional, default value = 1.0).
+        Reference liquid phase concentration.
+        The default is 1.0
 
     """
+
     adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
-    desorption_rate = DependentlySizedUnsignedList(dep='n_comp', default=1)
+    desorption_rate = DependentlySizedUnsignedList(dep='n_comp')
     characteristic_charge = DependentlySizedUnsignedList(dep='n_comp')
     steric_factor = DependentlySizedUnsignedList(dep='n_comp')
     capacity = UnsignedFloat()
-    reference_liquid_phase_conc = UnsignedFloat()
-    reference_solid_phase_conc = UnsignedFloat()
+    reference_liquid_phase_conc = UnsignedFloat(default=1.0)
+    reference_solid_phase_conc = UnsignedFloat(default=1.0)
 
     _parameter_names = BindingBaseClass._parameter_names + [
-            'adsorption_rate',
-            'desorption_rate',
-            'characteristic_charge',
-            'steric_factor',
-            'capacity',
-            'reference_liquid_phase_conc',
-            'reference_solid_phase_conc'
-        ]
+        'adsorption_rate',
+        'desorption_rate',
+        'characteristic_charge',
+        'steric_factor',
+        'capacity',
+        'reference_liquid_phase_conc',
+        'reference_solid_phase_conc'
+    ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'adsorption_rate',
+        'desorption_rate',
+        'characteristic_charge',
+        'steric_factor',
+        'capacity',
+    ]
+
+    @property
+    def adsorption_rate_transformed(self):
+        nu = np.array(self.characteristic_charge)
+        return \
+            self.adsorption_rate / \
+            self.reference_solid_phase_conc**(-nu)
+
+    @adsorption_rate_transformed.setter
+    def adsorption_rate_transformed(self, adsorption_rate_transformed):
+        nu = np.array(self.characteristic_charge)
+        self.adsorption_rate = \
+            adsorption_rate_transformed * \
+            self.reference_solid_phase_conc**(-nu)
+
+    @property
+    def desorption_rate_transformed(self):
+        nu = np.array(self.characteristic_charge)
+        return \
+            self.desorption_rate / \
+            self.reference_liquid_phase_conc**(-nu)
+
+    @desorption_rate_transformed.setter
+    def desorption_rate_transformed(self, desorption_rate_transformed):
+        nu = np.array(self.characteristic_charge)
+        self.desorption_rate = \
+            desorption_rate_transformed * \
+            self.reference_liquid_phase_conc**(-nu)
 
 
 class AntiLangmuir(BindingBaseClass):
@@ -317,6 +471,7 @@ class AntiLangmuir(BindingBaseClass):
         Anti-Langmuir coefficients. Length depends on n_comp.
 
     """
+
     adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
     desorption_rate = DependentlySizedUnsignedList(dep='n_comp')
     capacity = DependentlySizedUnsignedList(dep='n_comp')
@@ -328,51 +483,11 @@ class AntiLangmuir(BindingBaseClass):
         'capacity',
         'antilangmuir'
     ]
-
-    def __init__(self, *args, **kwargs):
-        self.adsorption_rate = [0.0] * self.n_comp
-        self.desorption_rate = [0.0] * self.n_comp
-        self.capacity = [0.0] * self.n_comp
-        self.antilangmuir = [0.0] * self.n_comp
-
-        super().__init__(*args, **kwargs)
-
-
-class KumarMultiComponentLangmuir(BindingBaseClass):
-    """Kumar Multi Component Langmuir adsoprtion isotherm.
-
-    Attributes
-    ----------
-    adsorption_rate : list of unsigned floats.
-        Adsorption rate constants.
-    desorption_rate : list of unsigned floats.
-        Desorption rate constants.
-    capacity : list of unsigned floats.
-        Maximum adsoprtion capacities.
-    characteristic_charge: list of unsigned floats.
-        Salt exponents/characteristic charges.
-    activation_temp : list of unsigned floats.
-        Activation temperatures.
-    temperature : unsigned float.
-        Temperature.
-
-    """
-    adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
-    desorption_rate = DependentlySizedUnsignedList(dep='n_comp', default=1)
-    capacity = DependentlySizedUnsignedList(dep='n_comp')
-    characteristic_charge = DependentlySizedUnsignedList(
-        dep='n_comp', default=1
-    )
-    activation_temp = DependentlySizedUnsignedList(dep='n_comp')
-    temperature = UnsignedFloat()
-
-    _parameter_names = BindingBaseClass._parameter_names + [
+    _required_parameters = BindingBaseClass._required_parameters + [
         'adsorption_rate',
         'desorption_rate',
         'capacity',
-        'characteristic_charge',
-        'activation_temp',
-        'temperature'
+        'antilangmuir'
     ]
 
 
@@ -391,11 +506,15 @@ class Spreading(BindingBaseClass):
         Exchange rates from the first to the second bound state.
     exchange_from_2_1 : list of unsigned floats.
         Exchange rates from the second to the first bound state.
+
     """
-    adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
-    desorption_rate = DependentlySizedUnsignedList(dep='n_comp', default=1)
-    capacity = DependentlySizedUnsignedList(dep='n_comp')
-    exchange_from_1_1 = DependentlySizedUnsignedList(dep='n_comp')
+
+    n_binding_sites = RangedInteger(lb=2, ub=2, default=2)
+
+    adsorption_rate = DependentlySizedUnsignedList(dep='n_total_bound')
+    desorption_rate = DependentlySizedUnsignedList(dep='n_total_bound')
+    capacity = DependentlySizedUnsignedList(dep='n_total_bound')
+    exchange_from_1_2 = DependentlySizedUnsignedList(dep='n_comp')
     exchange_from_2_1 = DependentlySizedUnsignedList(dep='n_comp')
 
     _parameter_names = BindingBaseClass._parameter_names + [
@@ -406,6 +525,14 @@ class Spreading(BindingBaseClass):
         'exchange_from_1_2',
         'exchange_from_2_1'
     ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'adsorption_rate',
+        'desorption_rate',
+        'activation_temp',
+        'capacity',
+        'exchange_from_1_2',
+        'exchange_from_2_1'
+    ]
 
 
 class MobilePhaseModulator(BindingBaseClass):
@@ -425,6 +552,7 @@ class MobilePhaseModulator(BindingBaseClass):
         Parameters describing the hydrophobicity (HIC).
 
     """
+
     adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
     desorption_rate = DependentlySizedUnsignedList(dep='n_comp')
     capacity = DependentlySizedUnsignedList(dep='n_comp')
@@ -440,6 +568,13 @@ class MobilePhaseModulator(BindingBaseClass):
         'ion_exchange_characteristic',
         'hydrophobicity',
     ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'adsorption_rate',
+        'desorption_rate',
+        'capacity',
+        'ion_exchange_characteristic',
+        'hydrophobicity',
+    ]
 
 
 class ExtendedMobilePhaseModulator(BindingBaseClass):
@@ -464,6 +599,7 @@ class ExtendedMobilePhaseModulator(BindingBaseClass):
         2 is modified Langmuir binding.
 
     """
+
     adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
     desorption_rate = DependentlySizedUnsignedList(dep='n_comp')
     capacity = DependentlySizedUnsignedList(dep='n_comp')
@@ -481,6 +617,14 @@ class ExtendedMobilePhaseModulator(BindingBaseClass):
         'hydrophobicity',
         'component_mode',
     ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'adsorption_rate',
+        'desorption_rate',
+        'capacity',
+        'ion_exchange_characteristic',
+        'hydrophobicity',
+        'component_mode',
+    ]
 
 
 class SelfAssociation(BindingBaseClass):
@@ -498,23 +642,26 @@ class SelfAssociation(BindingBaseClass):
         The characteristic charge v of the protein.
     steric_factor : list of unsigned floats.
         Steric factor of of the protein.
-    capacity : list of unsigned floats.
+    capacity : unsigned float.
         Stationary phase capacity (monovalent salt counterions); The total
         number of binding sites available on the resin surface.
-    reference_liquid_phase_conc : Parmater
-        Reference liquid phase concentration (optional, default value = 1.0).
-    reference_solid_phase_conc : Parmater
+    reference_liquid_phase_conc : unsigned float
         Reference liquid phase concentration (optional, default value = 1.0).
+        The default = 1.0
+    reference_solid_phase_conc : unsigned float
+        Reference liquid phase concentration (optional)
+        The default = 1.0
 
     """
+
     adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
     adsorption_rate_dimerization = DependentlySizedUnsignedList(dep='n_comp')
     desorption_rate = DependentlySizedUnsignedList(dep='n_comp')
     characteristic_charge = DependentlySizedUnsignedList(dep='n_comp')
     steric_factor = DependentlySizedUnsignedList(dep='n_comp')
-    capacity = DependentlySizedUnsignedList(dep='n_comp')
-    reference_liquid_phase_conc = DependentlySizedUnsignedList(dep='n_comp')
-    reference_solid_phase_conc = DependentlySizedUnsignedList(dep='n_comp')
+    capacity = UnsignedFloat()
+    reference_liquid_phase_conc = UnsignedFloat(default=1.0)
+    reference_solid_phase_conc = UnsignedFloat(default=1.0)
 
     _parameter_names = BindingBaseClass._parameter_names + [
         'adsorption_rate',
@@ -526,10 +673,18 @@ class SelfAssociation(BindingBaseClass):
         'reference_liquid_phase_conc',
         'reference_solid_phase_conc'
     ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'adsorption_rate',
+        'adsorption_rate_dimerization',
+        'desorption_rate',
+        'characteristic_charge',
+        'steric_factor',
+        'capacity',
+    ]
 
 
 class BiStericMassAction(BindingBaseClass):
-    """ Bi Steric Mass Action adsoprtion isotherm.
+    """Bi Steric Mass Action adsoprtion isotherm.
 
     Attributes
     ----------
@@ -543,34 +698,35 @@ class BiStericMassAction(BindingBaseClass):
     steric_factor : list of unsigned floats.
         Steric factor o (i,j) of the it-h protein with respect to the j-th
         binding site type in state-major ordering.
-    capacity : list of unsigned floats.
-        Stationary phase capacity (monovalent salt counterions): The total
-        number of binding site types.
+    capacity : unsigned float.
+        Stationary phase capacity (monovalent salt counterions); The total
+        number of binding sites available on the resin surface.
     reference_liquid_phase_conc : list of unsigned floats.
         Reference liquid phase concentration for each binding site type or one
-        value for all types (optional, default value = 1.0).
+        value for all types.
+        The default is 1.0
     reference_solid_phase_conc : list of unsigned floats.
         Reference solid phase concentration for each binding site type or one
-        value for all types (optional, default value = 1.0).
+        value for all types.
+        The default is 1.0
 
     """
-    adsorption_rate = DependentlySizedUnsignedList(
-        dep=('n_comp', 'n_states')
-    )
+
+    n_binding_sites = UnsignedInteger(default=2)
+
+    adsorption_rate = DependentlySizedUnsignedList(dep='n_bound_states')
     adsorption_rate_dimerization = DependentlySizedUnsignedList(
-        dep=('n_comp', 'n_states')
+        dep='n_bound_states'
     )
-    desorption_rate = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'))
-    characteristic_charge = DependentlySizedUnsignedList(
-        dep=('n_comp', 'n_states')
-    )
-    steric_factor = DependentlySizedUnsignedList(dep=('n_comp', 'n_states'))
-    capacity = DependentlySizedUnsignedList(dep='n_states')
+    desorption_rate = DependentlySizedUnsignedList(dep='n_bound_states')
+    characteristic_charge = DependentlySizedUnsignedList(dep='n_bound_states')
+    steric_factor = DependentlySizedUnsignedList(dep='n_bound_states')
+    capacity = DependentlySizedUnsignedList(dep='n_binding_sites')
     reference_liquid_phase_conc = DependentlySizedUnsignedList(
-        dep='n_states', default=1
+        dep='n_binding_sites', default=1
     )
     reference_solid_phase_conc = DependentlySizedUnsignedList(
-        dep='n_states', default=1
+        dep='n_binding_sites', default=1
     )
 
     _parameter_names = BindingBaseClass._parameter_names + [
@@ -582,6 +738,13 @@ class BiStericMassAction(BindingBaseClass):
         'reference_liquid_phase_conc',
         'reference_solid_phase_conc'
     ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'adsorption_rate',
+        'desorption_rate',
+        'characteristic_charge',
+        'steric_factor',
+        'capacity',
+    ]
 
     def __init__(self, *args, n_states=2, **kwargs):
         self.n_states = n_states
@@ -589,7 +752,7 @@ class BiStericMassAction(BindingBaseClass):
 
 
 class MultistateStericMassAction(BindingBaseClass):
-    """ Multistate Steric Mass Action adsoprtion isotherm.
+    """Multistate Steric Mass Action adsoprtion isotherm.
 
     Attributes
     ----------
@@ -608,28 +771,32 @@ class MultistateStericMassAction(BindingBaseClass):
     conversion_rate : list of unsigned floats.
         Conversion rates between different bound states in
         component-major ordering.
-    capacity : list of unsigned floats.
-        Stationary phase capacity (monovalent salt counterions): The total
+        Length: $sum_{i=1}^{n_{comp}} n_{bound, i}$
+    capacity : unsigned float.
+        Stationary phase capacity (monovalent salt counterions); The total
         number of binding sites available on the resin surface.
-    reference_liquid_phase_conc : list of unsigned floats.
-        Reference liquid phase concentration (optional, default value = 1.0).
-    reference_solid_phase_conc : list of unsigned floats.
-        Reference solid phase concentration (optional, default value = 1.0).
+    reference_liquid_phase_conc : unsigned float.
+        Reference liquid phase concentration.
+        The default = 1.0
+    reference_solid_phase_conc : unsigned float, optional
+        Reference solid phase concentration.
+        The default = 1.0
 
     """
-    adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
-    desorption_rate = DependentlySizedUnsignedList(dep='n_comp', default=1)
-    characteristic_charge = DependentlySizedUnsignedList(dep='n_comp')
-    steric_factor = DependentlySizedUnsignedList(dep='n_comp')
-    conversion_rate = DependentlySizedUnsignedList(dep='n_comp')
-    capacity = DependentlySizedUnsignedList(dep='n_comp')
-    reference_liquid_phase_conc = DependentlySizedUnsignedList(
-        dep='n_comp', default=1
-    )
-    reference_solid_phase_conc = DependentlySizedUnsignedList(
-        dep='n_comp', default=1
+
+    bound_states = DependentlySizedUnsignedIntegerList(
+        dep=('n_binding_sites', 'n_comp'), default=1
     )
 
+    adsorption_rate = DependentlySizedUnsignedList(dep='n_bound_states')
+    desorption_rate = DependentlySizedUnsignedList(dep='n_bound_states')
+    characteristic_charge = DependentlySizedUnsignedList(dep='n_bound_states')
+    steric_factor = DependentlySizedUnsignedList(dep='n_bound_states')
+    conversion_rate = DependentlySizedUnsignedList(dep='_conversion_entries')
+    capacity = UnsignedFloat()
+    reference_liquid_phase_conc = UnsignedFloat(default=1)
+    reference_solid_phase_conc = UnsignedFloat(default=1)
+
     _parameter_names = BindingBaseClass._parameter_names + [
         'adsorption_rate',
         'desorption_rate',
@@ -640,10 +807,26 @@ class MultistateStericMassAction(BindingBaseClass):
         'reference_liquid_phase_conc',
         'reference_solid_phase_conc'
     ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'adsorption_rate',
+        'desorption_rate',
+        'characteristic_charge',
+        'steric_factor',
+        'conversion_rate',
+        'capacity',
+    ]
+
+    @property
+    def _conversion_entries(self):
+        n = 0
+        for state in self.bound_states[1:]:
+            n += state**2
+
+        return n
 
 
-class SimplifiedMultistateSteric_Mass_Action(BindingBaseClass):
-    """ Simplified multistate Steric Mass Action adsoprtion isotherm.
+class SimplifiedMultistateStericMassAction(BindingBaseClass):
+    """Simplified multistate Steric Mass Action adsoprtion isotherm.
 
     Attributes
     ----------
@@ -669,7 +852,7 @@ class SimplifiedMultistateSteric_Mass_Action(BindingBaseClass):
     quadratic_modifiers_steric : list of unsigned floats.
         Quadratic modifiers of the sterif factors of the different components
         depending on the index of the bound state.
-    capacity : list of unsigned floats.
+    capacity : unsigned floats.
         Stationary phase capacity (monovalent salt counterions): The total
         number of binding sites available on the resin surface.
     exchange_from_weak_stronger : list of unsigned floats.
@@ -696,23 +879,28 @@ class SimplifiedMultistateSteric_Mass_Action(BindingBaseClass):
         Reference solid phase concentration (optional, default value = 1.0).
 
     """
-    adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
-    desorption_rate = DependentlySizedUnsignedList(dep='n_comp')
+
+    bound_states = DependentlySizedUnsignedIntegerList(
+        dep=('n_binding_sites', 'n_comp'), default=1
+    )
+
+    adsorption_rate = DependentlySizedUnsignedList(dep='n_bound_states')
+    desorption_rate = DependentlySizedUnsignedList(dep='n_bound_states')
     characteristic_charge_first = DependentlySizedUnsignedList(dep='n_comp')
     characteristic_charge_last = DependentlySizedUnsignedList(dep='n_comp')
     quadratic_modifiers_charge = DependentlySizedUnsignedList(dep='n_comp')
     steric_factor_first = DependentlySizedUnsignedList(dep='n_comp')
     steric_factor_last = DependentlySizedUnsignedList(dep='n_comp')
     quadratic_modifiers_steric = DependentlySizedUnsignedList(dep='n_comp')
-    capacity = DependentlySizedUnsignedList(dep='n_comp')
+    capacity = UnsignedFloat()
     exchange_from_weak_stronger = DependentlySizedUnsignedList(dep='n_comp')
     linear_exchange_ws = DependentlySizedUnsignedList(dep='n_comp')
     quadratic_exchange_ws = DependentlySizedUnsignedList(dep='n_comp')
     exchange_from_stronger_weak = DependentlySizedUnsignedList(dep='n_comp')
     linear_exchange_sw = DependentlySizedUnsignedList(dep='n_comp')
     quadratic_exchange_sw = DependentlySizedUnsignedList(dep='n_comp')
-    reference_liquid_phase_conc = DependentlySizedUnsignedList(dep='n_comp')
-    reference_solid_phase_conc = DependentlySizedUnsignedList(dep='n_comp')
+    reference_liquid_phase_conc = UnsignedFloat(default=1)
+    reference_solid_phase_conc = UnsignedFloat(default=1)
 
     _parameter_names = BindingBaseClass._parameter_names + [
         'adsorption_rate',
@@ -733,6 +921,23 @@ class SimplifiedMultistateSteric_Mass_Action(BindingBaseClass):
         'reference_liquid_phase_conc',
         'reference_solid_phase_conc'
     ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'adsorption_rate',
+        'desorption_rate',
+        'characteristic_charge_first',
+        'characteristic_charge_last',
+        'quadratic_modifiers_charge',
+        'steric_factor_first',
+        'steric_factor_last',
+        'quadratic_modifiers_steric',
+        'capacity',
+        'exchange_from_weak_stronger',
+        'linear_exchange_ws',
+        'quadratic_exchange_ws',
+        'exchange_from_stronger_weak',
+        'linear_exchange_sw',
+        'quadratic_exchange_sw',
+    ]
 
 
 class Saska(BindingBaseClass):
@@ -746,12 +951,16 @@ class Saska(BindingBaseClass):
         Quadratic factors.
 
     """
+
     henry_const = DependentlySizedUnsignedList(dep='n_comp')
-    quadratic_factor = DependentlySizedUnsignedList(dep='n_comp')
+    quadratic_factor = DependentlySizedUnsignedList(dep=('n_comp', 'n_comp'))
 
     _parameter_names = BindingBaseClass._parameter_names + [
         'henry_const', 'quadratic_factor'
     ]
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'henry_const', 'quadratic_factor'
+    ]
 
 
 class GeneralizedIonExchange(BindingBaseClass):
@@ -811,55 +1020,58 @@ class GeneralizedIonExchange(BindingBaseClass):
         Reference liquid phase concentration (optional, default value = 1.0).
 
     """
-    adsorption_rate = DependentlySizedUnsignedList(dep='n_comp')
-    adsorption_rate_linear = DependentlySizedUnsignedList(dep='n_comp')
-    adsorption_rate_quadratic = DependentlySizedUnsignedList(
+
+    non_binding_component_indices = [1]
+
+    adsorption_rate = DependentlySizedList(dep='n_comp')
+    adsorption_rate_linear = DependentlySizedList(dep='n_comp')
+    adsorption_rate_quadratic = DependentlySizedList(
         dep='n_comp', default=0
     )
-    adsorption_rate_cubic = DependentlySizedUnsignedList(
+    adsorption_rate_cubic = DependentlySizedList(
         dep='n_comp', default=0
     )
-    adsorption_rate_salt = DependentlySizedUnsignedList(
+    adsorption_rate_salt = DependentlySizedList(
         dep='n_comp', default=0
     )
-    adsorption_rate_protein = DependentlySizedUnsignedList(
+    adsorption_rate_protein = DependentlySizedList(
         dep='n_comp', default=0
     )
-    desorption_rate = DependentlySizedUnsignedList(dep='n_comp', default=1)
-    desorption_rate_linear = DependentlySizedUnsignedList(
+    desorption_rate = DependentlySizedList(dep='n_comp')
+    desorption_rate_linear = DependentlySizedList(
         dep='n_comp', default=0
     )
-    desorption_rate_quadratic = DependentlySizedUnsignedList(
+    desorption_rate_quadratic = DependentlySizedList(
         dep='n_comp', default=0
     )
-    desorption_rate_cubic = DependentlySizedUnsignedList(
+    desorption_rate_cubic = DependentlySizedList(
         dep='n_comp', default=0
     )
-    desorption_rate_salt = DependentlySizedUnsignedList(
+    desorption_rate_salt = DependentlySizedList(
         dep='n_comp', default=0
     )
-    desorption_rate_protein = DependentlySizedUnsignedList(
+    desorption_rate_protein = DependentlySizedList(
         dep='n_comp', default=0
     )
-    characteristic_charge = DependentlySizedUnsignedList(
-        dep='_break_length', default=1
+    characteristic_charge_breaks = DependentlyModulatedUnsignedList(
+        dep='n_comp'
     )
-    characteristic_charge_linear = DependentlySizedUnsignedList(
-        dep='characteristic_charge', default=0
+    characteristic_charge = DependentlySizedList(
+        dep=('n_pieces', 'n_comp'),
     )
-    characteristic_charge_quadratic = DependentlySizedUnsignedList(
-        dep='characteristic_charge', default=0
+    characteristic_charge_linear = DependentlySizedList(
+        dep=('n_pieces', 'n_comp'), default=0
     )
-    characteristic_charge_cubic = DependentlySizedUnsignedList(
-        dep='characteristic_charge', default=0
+    characteristic_charge_quadratic = DependentlySizedList(
+        dep=('n_pieces', 'n_comp'), default=0
     )
-    characteristic_charge_breaks = DependentlySizedUnsignedList(
-        dep='_break_length', default=0
+    characteristic_charge_cubic = DependentlySizedList(
+        dep=('n_pieces', 'n_comp'), default=0
     )
     steric_factor = DependentlySizedUnsignedList(dep='n_comp')
     capacity = UnsignedFloat()
-    reference_liquid_phase_conc = UnsignedFloat()
-    reference_solid_phase_conc = UnsignedFloat()
+    reference_liquid_phase_conc = UnsignedFloat(default=1)
+    reference_solid_phase_conc = UnsignedFloat(default=1)
 
     _parameter_names = BindingBaseClass._parameter_names + [
         'adsorption_rate',
@@ -874,24 +1086,31 @@ class GeneralizedIonExchange(BindingBaseClass):
         'desorption_rate_cubic',
         'desorption_rate_salt',
         'desorption_rate_protein',
+        'characteristic_charge_breaks',
         'characteristic_charge',
         'characteristic_charge_linear',
         'characteristic_charge_quadratic',
         'characteristic_charge_cubic',
-        'characteristic_charge_breaks',
         'steric_factor',
         'capacity',
         'reference_liquid_phase_conc',
         'reference_solid_phase_conc',
     ]
-
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+    _required_parameters = BindingBaseClass._required_parameters + [
+        'adsorption_rate',
+        'desorption_rate',
+        'characteristic_charge',
+        'steric_factor',
+        'capacity',
+    ]
 
     @property
-    def _multiples(self):
-        pass
+    def n_pieces(self):
+        """int: Number of pieces for cubic polynomial description of nu."""
+        if self.characteristic_charge_breaks is None:
+            return 1
 
-    @property
-    def _break_length(self):
-        return len(self.characteristic_charge) + 1
+        n_pieces_all = len(self.characteristic_charge_breaks) - self.n_comp
+        n_pieces_comp = int(n_pieces_all / self.n_comp)
+
+        return n_pieces_comp
