@@ -1,5 +1,6 @@
 from collections import defaultdict
 from functools import wraps
+from warnings import warn
 
 import numpy as np
 import sympy as sym
@@ -10,7 +11,7 @@ from CADETProcess.dataStructure import frozen_attributes
 from CADETProcess.dataStructure import StructMeta, UnsignedInteger, String
 from .componentSystem import ComponentSystem
 from .unitOperation import UnitBaseClass
-from .unitOperation import Source, SourceMixin, Sink, SinkMixin, Cstr
+from .unitOperation import Inlet, Outlet, Cstr
 from .binding import NoBinding
 
 
@@ -43,9 +44,9 @@ class FlowSheet(metaclass=StructMeta):
         self.component_system = component_system
         self.name = name
         self._units = []
-        self._feed_sources = []
-        self._eluent_sources = []
-        self._chromatogram_sinks = []
+        self._feed_inlets = []
+        self._eluent_inlets = []
+        self._product_outlets = []
         self._connections = Dict()
         self._output_states = Dict()
         self._flow_rates = Dict()
@@ -173,14 +174,19 @@ class FlowSheet(metaclass=StructMeta):
         return self.units.index(unit)
 
     @property
-    def sources(self):
-        """list: All UnitOperations implementing the SourceMixin interface."""
-        return [unit for unit in self._units if isinstance(unit, SourceMixin)]
+    def inlets(self):
+        """list: All Inlets in the system."""
+        return [unit for unit in self._units if isinstance(unit, Inlet)]
 
     @property
-    def sinks(self):
-        """list: All UnitOperations implementing the SinkMixin interface."""
-        return [unit for unit in self._units if isinstance(unit, SinkMixin)]
+    def outlets(self):
+        """list: All Outlets in the system."""
+        return [unit for unit in self._units if isinstance(unit, Outlet)]
+
+    @property
+    def cstrs(self):
+        """list: All Cstrs in the system."""
+        return [unit for unit in self._units if isinstance(unit, Cstr)]
 
     @property
     def units_with_binding(self):
@@ -191,19 +197,19 @@ class FlowSheet(metaclass=StructMeta):
     @update_parameters_decorator
     def add_unit(
             self, unit,
-            feed_source=False, eluent_source=False, chromatogram_sink=False):
+            feed_inlet=False, eluent_inlet=False, product_outlet=False):
         """Add unit to the flow sheet.
 
         Parameters
         ----------
         unit : UnitBaseClass
             UnitBaseClass object to be added to the flow sheet.
-        feed_source : bool
-            If True, add unit to feed sources.
-        eluent_source : bool
-            If True, add unit to eluent sources.
-        chromatogram_sink : bool
-            If True, add unit to chromatogram sinks.
+        feed_inlet : bool
+            If True, add unit to feed inlets.
+        eluent_inlet : bool
+            If True, add unit to eluent inlets.
+        product_outlet : bool
+            If True, add unit to product outlets.
 
         Raises
         ------
@@ -221,7 +227,7 @@ class FlowSheet(metaclass=StructMeta):
         if not isinstance(unit, UnitBaseClass):
             raise TypeError('Expected UnitOperation')
 
-        if unit in self._units:
+        if unit in self._units or unit.name in self.unit_names:
             raise CADETProcessError('Unit already part of System')
 
         if unit.component_system is not self.component_system:
@@ -237,12 +243,12 @@ class FlowSheet(metaclass=StructMeta):
 
         super().__setattr__(unit.name, unit)
 
-        if feed_source:
-            self.add_feed_source(unit)
-        if eluent_source:
-            self.add_eluent_source(unit)
-        if chromatogram_sink:
-            self.add_chromatogram_sink(unit)
+        if feed_inlet:
+            self.add_feed_inlet(unit)
+        if eluent_inlet:
+            self.add_eluent_inlet(unit)
+        if product_outlet:
+            self.add_product_outlet(unit)
 
     @unit_name_decorator
     @update_parameters_decorator
@@ -252,8 +258,8 @@ class FlowSheet(metaclass=StructMeta):
         Removes unit from the list. Tries to remove units which are twice
         located as desinations. For this the origins and destinations are
         deleted for the unit. Raises a CADETProcessError if an ValueError is
-        excepted. If the unit is specified as feed_source, eluent_source
-        or chromatogram_sink, the corresponding attributes are deleted.
+        excepted. If the unit is specified as feed_inlet, eluent_inlet
+        or product_outlet, the corresponding attributes are deleted.
 
         Parameters
         ----------
@@ -268,20 +274,20 @@ class FlowSheet(metaclass=StructMeta):
         See Also
         --------
         add_unit
-        feed_source
-        eluent_source
-        chromatogram_sink
+        feed_inlet
+        eluent_inlet
+        product_outlet
 
         """
         if unit not in self.units:
             raise CADETProcessError('Unit not in flow sheet')
 
-        if unit is self.feed_sources:
-            self.remove_feed_source(unit)
-        if unit is self.eluent_sources:
-            self.remove_eluent_source(unit)
-        if unit is self.chromatogram_sinks:
-            self.remove_chromatogram_sink(unit)
+        if unit is self.feed_inlets:
+            self.remove_feed_inlet(unit)
+        if unit is self.eluent_inlets:
+            self.remove_eluent_inlet(unit)
+        if unit is self.product_outlets:
+            self.remove_product_outlet(unit)
 
         origins = self.connections[unit].origins.copy()
         for origin in origins:
@@ -404,42 +410,72 @@ class FlowSheet(metaclass=StructMeta):
 
         Raises
         ------
-        CADETProcessError
+        Warning
             If Inlets have ingoing streams.
-            If Outlets have outgoint streams.
+            If Outlets have outgoing streams.
             If Units (other than Cstr) are not fully connected.
 
+        Returns
+        -------
+        flag : bool
+            True if all units are connected correctly. False otherwise.
+
         """
+        flag = True
         for unit, connections in self.connections.items():
-            if isinstance(unit, Source):
+            if isinstance(unit, Inlet):
                 if len(connections.origins) != 0:
-                    raise CADETProcessError(
-                        "Inlet unit cannot have ingoing stream."
-                    )
+                    flag = False
+                    warn("Inlet unit cannot have ingoing stream.")
                 if len(connections.destinations) == 0:
-                    raise CADETProcessError(
-                        f" Unit '{unit.name}' does not have outgoing stream."
-                    )
-            elif isinstance(unit, Sink):
+                    flag = False
+                    warn(f" Unit '{unit.name}' does not have outgoing stream.")
+            elif isinstance(unit, Outlet):
                 if len(connections.destinations) != 0:
-                    raise CADETProcessError(
-                        "Outlet unit cannot have outgoing stream."
-                    )
+                    flag = False
+                    warn("Outlet unit cannot have outgoing stream.")
                 if len(connections.origins) == 0:
-                    raise CADETProcessError(
-                        f"Unit '{unit.name}' does not have ingoing stream."
-                    )
+                    flag = False
+                    warn(f"Unit '{unit.name}' does not have ingoing stream.")
             elif isinstance(unit, Cstr):
-                continue
+                if unit.flow_rate is None and len(connections.destinations) == 0:
+                    flag = False
+                    warn("Cstr cannot have flow rate without outgoing stream.")
             else:
                 if len(connections.origins) == 0:
-                    raise CADETProcessError(
-                        f"Unit '{unit.name}' does not have ingoing stream."
-                    )
+                    flag = False
+                    warn(f"Unit '{unit.name}' does not have ingoing stream.")
                 if len(connections.destinations) == 0:
-                    raise CADETProcessError(
-                        f" Unit '{unit.name}' does not have outgoing stream."
-                    )
+                    flag = False
+                    warn(f" Unit '{unit.name}' does not have outgoing stream.")
+
+        return flag
+
+    @property
+    def missing_parameters(self):
+        missing_parameters = []
+
+        for unit in self.units:
+            missing_parameters += [
+                f'{unit.name}.{param}' for param in unit.missing_parameters
+            ]
+
+        return missing_parameters
+
+    def check_units_config(self):
+        """Check if units are configured correctly.
+
+        Returns
+        -------
+        flag : bool
+            True if units are configured correctly. False otherwise.
+
+        """
+        flag = True
+        for unit in self.units:
+            if not unit.check_required_parameters():
+                flag = False
+        return flag
 
     @property
     def output_states(self):
@@ -454,7 +490,7 @@ class FlowSheet(metaclass=StructMeta):
         ----------
         unit : UnitBaseClass
             UnitOperation of flowsheet.
-        state : int or list of floats
+        state : int or list of floats or dict
             new output state of the unit.
 
         Raises
@@ -481,21 +517,36 @@ class FlowSheet(metaclass=StructMeta):
             output_state = [0] * state_length
             output_state[state] = 1
 
-        else:
+        elif isinstance(state, dict):
+            output_state = [0] * state_length
+            for dest, value in state.items():
+                try:
+                    assert self.connection_exists(unit, dest)
+                except AssertionError:
+                    raise CADETProcessError(f'{unit} does not connect to {dest}.')
+                dest = self[dest]
+                index = self.connections[unit].destinations.index(dest)
+                output_state[index] = value
+
+        elif isinstance(state, list):
             if len(state) != state_length:
                 raise CADETProcessError(f'Expected length {state_length}.')
 
-            elif not np.isclose(sum(state), 1):
-                raise CADETProcessError('Sum of fractions must be 1')
-
             output_state = state
 
+        else:
+            raise TypeError("Output state must be integer, list or dict.")
+
+        if not np.isclose(sum(output_state), 1):
+            raise CADETProcessError('Sum of fractions must be 1')
+
         self._output_states[unit] = output_state
 
     def get_flow_rates(self, state=None):
-        """Calculate flow rate for all connections.unit operation flow rates.
+        """Calculate flow rate for all connections.
 
-        If an additional state is passed, it will b
+        Optionally, an additional output state can be passed to update the
+        current output states.
 
         Parameters
         ----------
@@ -509,8 +560,10 @@ class FlowSheet(metaclass=StructMeta):
 
         """
         flow_rates = {
-            unit.name: unit.flow_rate for unit in self.sources
+            unit.name: unit.flow_rate for unit in (self.inlets + self.cstrs)
+            if unit.flow_rate is not None
         }
+
         output_states = self.output_states
 
         if state is not None:
@@ -554,12 +607,12 @@ class FlowSheet(metaclass=StructMeta):
                 flow_rates[unit.name].origins[origin] = np.array(flow_rate)
 
         for unit in self.units:
-            if not isinstance(unit, Source):
+            if not isinstance(unit, Inlet):
                 flow_rate_in = np.sum(
                     list(flow_rates[unit.name].origins.values()), axis=0
                 )
                 flow_rates[unit.name].total_in = flow_rate_in
-            if not isinstance(unit, Sink):
+            if not isinstance(unit, Outlet):
                 flow_rate_out = np.sum(
                     list(flow_rates[unit.name].destinations.values()), axis=0
                 )
@@ -567,7 +620,7 @@ class FlowSheet(metaclass=StructMeta):
 
         return flow_rates
 
-    def solve_flow_rates(self, source_flow_rates, output_states, coeff=0):
+    def solve_flow_rates(self, inlet_flow_rates, output_states, coeff=0):
         """Solve flow rates of system using sympy.
 
         Because a simple 'push' algorithm cannot be used when closed loops are
@@ -576,8 +629,8 @@ class FlowSheet(metaclass=StructMeta):
 
         Parameters
         ----------
-        source_flow_rates: dict
-            Flow rates of Source UnitOperations.
+        inlet_flow_rates: dict
+            Flow rates of Inlet UnitOperations.
         output_states: dict
             Output states of all UnitOperations.
         coeff: int
@@ -592,12 +645,18 @@ class FlowSheet(metaclass=StructMeta):
         -----
             Since dynamic flow rates can be described as cubic polynomials, the
             flow rates are solved individually for all coefficients.
+            To comply to the interface, the flow rates are always computed for the first
+            (constant) coefficient. For higher orders, the function returns None if all
+            coefficients are zero.
 
+            The problem could definitely be solved more elegantly (and efficiently) by
+            using proper liner algebra.
         """
-        coeffs = np.array(
-            [source_flow_rates[unit.name][coeff] for unit in self.sources]
-        )
-        if not np.any(coeffs):
+        if len(inlet_flow_rates) == 0:
+            return None
+
+        coeffs = np.array(list(inlet_flow_rates.values()), ndmin=2)[:, coeff]
+        if coeff > 0 and not np.any(coeffs):
             return None
 
         # Setup lists for symbols
@@ -612,11 +671,17 @@ class FlowSheet(metaclass=StructMeta):
 
         # Setup symbolic equations
         for unit_index, unit in enumerate(self.units):
-            if isinstance(unit, SourceMixin):
+            if \
+                    isinstance(unit, Inlet) or \
+                    isinstance(unit, Cstr) and (
+                        unit.flow_rate is not None
+                        or
+                        unit.name in inlet_flow_rates
+                    ):
                 unit_total_flow_eq.append(
                     sym.Add(
                         unit_total_flow_symbols[unit_index],
-                        - float(source_flow_rates[unit.name][coeff])
+                        - float(inlet_flow_rates[unit.name][coeff])
                     )
                 )
             else:
@@ -637,7 +702,7 @@ class FlowSheet(metaclass=StructMeta):
                 unit_inflow_symbols += unit_i_inflow_symbols
                 unit_total_flow_eq.append(unit_i_total_flow_eq)
 
-            if not isinstance(unit, Sink):
+            if not isinstance(unit, Outlet):
                 output_state = output_states[unit]
                 unit_i_outflow_symbols = []
 
@@ -674,157 +739,160 @@ class FlowSheet(metaclass=StructMeta):
     def check_flow_rates(self, state=None):
         flow_rates = self.get_flow_rates(state)
         for unit, q in flow_rates.items():
-            if isinstance(unit, (SourceMixin, SinkMixin)):
+            if isinstance(unit, (Inlet, Outlet)):
                 continue
+            elif isinstance(unit, Cstr) and Cstr.flow_rate is not None:
+                continue
+
             if not np.all(q.total_in == q.total_out):
                 raise CADETProcessError(
                     f"Unbalanced flow rate for unit '{unit}'."
                 )
 
     @property
-    def feed_sources(self):
-        """list: Sources considered for calculating recovery yield."""
-        return self._feed_sources
+    def feed_inlets(self):
+        """list: Inlets considered for calculating recovery yield."""
+        return self._feed_inlets
 
     @unit_name_decorator
-    def add_feed_source(self, feed_source):
-        """Add source to list of units to be considered for recovery.
+    def add_feed_inlet(self, feed_inlet):
+        """Add inlet to list of units to be considered for recovery.
 
         Parameters
         ----------
-        feed_source : SourceMixin
-            Unit to be added to list of feed sources
+        feed_inlet : SourceMixin
+            Unit to be added to list of feed inlets.
 
         Raises
         ------
         CADETProcessError
-            If unit is not in a source object
-            If unit is already marked as feed source
+            If unit is not an Inlet.
+            If unit is already marked as feed inlet.
 
         """
-        if feed_source not in self.sources:
-            raise CADETProcessError('Expected Source')
-        if feed_source in self._feed_sources:
+        if feed_inlet not in self.inlets:
+            raise CADETProcessError('Expected Inlet')
+        if feed_inlet in self._feed_inlets:
             raise CADETProcessError(
-                f'Unit \'{feed_source}\' is already a feed source'
+                f'Unit \'{feed_inlet}\' is already a feed inlet.'
             )
-        self._feed_sources.append(feed_source)
+        self._feed_inlets.append(feed_inlet)
 
     @unit_name_decorator
-    def remove_feed_source(self, feed_source):
-        """Remove source from list of units to be considered for recovery.
+    def remove_feed_inlet(self, feed_inlet):
+        """Remove inlet from list of units to be considered for recovery.
 
         Parameters
         ----------
-        feed_source : SourceMixin
-            Unit to be removed from list of feed sources.
+        feed_inlet : SourceMixin
+            Unit to be removed from list of feed inlets.
 
         """
-        if feed_source not in self._feed_sources:
+        if feed_inlet not in self._feed_inlets:
             raise CADETProcessError(
-                f'Unit \'{feed_source}\' is not a feed source.'
+                f'Unit \'{feed_inlet}\' is not a feed inlet.'
             )
-        self._feed_sources.remove(feed_source)
+        self._feed_inlets.remove(feed_inlet)
 
     @property
-    def eluent_sources(self):
-        """list: Sources to be considered for eluent consumption."""
-        return self._eluent_sources
+    def eluent_inlets(self):
+        """list: Inlets to be considered for eluent consumption."""
+        return self._eluent_inlets
 
     @unit_name_decorator
-    def add_eluent_source(self, eluent_source):
-        """Add source to list of units to be considered for eluent consumption.
+    def add_eluent_inlet(self, eluent_inlet):
+        """Add inlet to list of units to be considered for eluent consumption.
 
         Parameters
         ----------
-        eluent_source : SourceMixin
-            Unit to be added to list of eluent sources.
+        eluent_inlet : SourceMixin
+            Unit to be added to list of eluent inlets.
 
         Raises
         ------
         CADETProcessError
-            If unit is not in a source object
-            If unit is already marked as eluent source
+            If unit is not an Inlet.
+            If unit is already marked as eluent inlet.
 
         """
-        if eluent_source not in self.sources:
-            raise CADETProcessError('Expected Source')
-        if eluent_source in self._eluent_sources:
+        if eluent_inlet not in self.inlets:
+            raise CADETProcessError('Expected Inlet')
+        if eluent_inlet in self._eluent_inlets:
             raise CADETProcessError(
-                f'Unit \'{eluent_source}\' is already an eluent source'
+                f'Unit \'{eluent_inlet}\' is already an eluent inlet'
             )
-        self._eluent_sources.append(eluent_source)
+        self._eluent_inlets.append(eluent_inlet)
 
     @unit_name_decorator
-    def remove_eluent_source(self, eluent_source):
-        """Remove source from list of units considered for eluent consumption.
+    def remove_eluent_inlet(self, eluent_inlet):
+        """Remove inlet from list of units considered for eluent consumption.
 
         Parameters
         ----------
-        eluent_source : SourceMixin
-            Unit to be added to list of eluent sources.
+        eluent_inlet : SourceMixin
+            Unit to be added to list of eluent inlets.
 
         Raises
         ------
         CADETProcessError
-            If unit is not in eluent sources
+            If unit is not in eluent inlets.
 
         """
-        if eluent_source not in self._eluent_sources:
+        if eluent_inlet not in self._eluent_inlets:
             raise CADETProcessError(
-                f'Unit \'{eluent_source}\' is not an eluent source.'
+                f'Unit \'{eluent_inlet}\' is not an eluent inlet.'
             )
-        self._eluent_sources.remove(eluent_source)
+        self._eluent_inlets.remove(eluent_inlet)
 
     @property
-    def chromatogram_sinks(self):
-        """list: Sinks to be considered for fractionation."""
-        return self._chromatogram_sinks
+    def product_outlets(self):
+        """list: Outlets to be considered for fractionation."""
+        return self._product_outlets
 
     @unit_name_decorator
-    def add_chromatogram_sink(self, chromatogram_sink):
-        """Add sink to list of units considered for fractionation.
+    def add_product_outlet(self, product_outlet):
+        """Add outlet to list of units considered for fractionation.
 
         Parameters
         ----------
-        chromatogram_sink : SinkMixin
-            Unit to be added to list of chromatogram sinks.
+        product_outlet : Outlet
+            Unit to be added to list of product outlets.
 
         Raises
         ------
         CADETProcessError
-            If unit is not a sink object.
-            If unit is already marked as chromatogram sink.
+            If unit is not an Outlet.
+            If unit is already marked as product outlet.
 
         """
-        if chromatogram_sink not in self.sinks:
-            raise CADETProcessError('Expected Sink')
-        if chromatogram_sink in self._chromatogram_sinks:
+        if product_outlet not in self.outlets:
+            raise CADETProcessError('Expected Outlet')
+        if product_outlet in self._product_outlets:
             raise CADETProcessError(
-                f'Unit \'{chromatogram_sink}\' is already an chomatogram sink'
+                f'Unit \'{product_outlet}\' is already a product outlet'
             )
-        self._chromatogram_sinks.append(chromatogram_sink)
+        self._product_outlets.append(product_outlet)
 
     @unit_name_decorator
-    def remove_chromatogram_sink(self, chromatogram_sink):
-        """Remove sink from list of units to be considered for fractionation.
+    def remove_product_outlet(self, product_outlet):
+        """Remove outlet from list of units to be considered for fractionation.
 
         Parameters
         ----------
-        chromatogram_sink : SinkMixin
-            Unit to be added to list of chromatogram sinks.
+        product_outlet : Outlet
+            Unit to be added to list of product outlets.
 
         Raises
         ------
         CADETProcessError
-            If unit is not a chromatogram sink.
+            If unit is not a product outlet.
 
         """
-        if chromatogram_sink not in self._chromatogram_sinks:
+        if product_outlet not in self._product_outlets:
             raise CADETProcessError(
-                f'Unit \'{chromatogram_sink}\' is not a chromatogram sink.'
+                f'Unit \'{product_outlet}\' is not a product outlet.'
             )
-        self._chromatogram_sinks.remove(chromatogram_sink)
+        self._product_outlets.remove(product_outlet)
 
     @property
     def parameters(self):
