@@ -1,5 +1,6 @@
 from abc import abstractproperty
 import math
+from warnings import warn
 
 from CADETProcess import CADETProcessError
 
@@ -18,16 +19,30 @@ from .reaction import ReactionBaseClass, NoReaction
 from .discretization import (
     DiscretizationParametersBase, NoDiscretization,
     LRMDiscretizationFV, LRMDiscretizationDG,
-    LRMPDiscretizationFV, GRMDiscretizationFV
+    LRMPDiscretizationFV, LRMPDiscretizationDG,
+    GRMDiscretizationFV, GRMDiscretizationDG
 )
 
 from .solutionRecorder import (
-    SolutionRecorderIO,
-    TubularReactorRecorder, LRMRecorder, LRMPRecorder, GRMRecorder, 
-    CSTRRecorder
+    IORecorder,
+    TubularReactorRecorder, LRMRecorder, LRMPRecorder, GRMRecorder, CSTRRecorder
 )
 
 
+__all__ = [
+    'UnitBaseClass',
+    'SourceMixin',
+    'SinkMixin',
+    'Inlet',
+    'Outlet',
+    'TubularReactorBase',
+    'TubularReactor',
+    'LumpedRateModelWithoutPores',
+    'LumpedRateModelWithPores',
+    'GeneralRateModel',
+]
+
+
 @frozen_attributes
 class UnitBaseClass(metaclass=StructMeta):
     """Base class for all UnitOperation classes.
@@ -44,9 +59,11 @@ class UnitBaseClass(metaclass=StructMeta):
     parameters : list
         list of parameter names.
     name : String
-        name of the unit_operation.
+        name of the unit operation.
     binding_model : BindingBaseClass
         binding behavior of the unit. Defaults to NoBinding.
+    solution_recorder : IORecorder
+        Solution recorder for the unit operation.
 
     See Also
     --------
@@ -61,6 +78,7 @@ class UnitBaseClass(metaclass=StructMeta):
     _section_dependent_parameters = []
     _polynomial_parameters = []
     _initial_state = []
+    _required_parameters = []
 
     supports_bulk_reaction = False
     supports_particle_reaction = False
@@ -77,7 +95,7 @@ class UnitBaseClass(metaclass=StructMeta):
 
         self.discretization = NoDiscretization()
 
-        self.solution_recorder = SolutionRecorderIO()
+        self.solution_recorder = IORecorder()
 
         self._parameters = {
             param: getattr(self, param)
@@ -184,6 +202,10 @@ class UnitBaseClass(metaclass=StructMeta):
         }
         return parameters
 
+    @property
+    def required_parameters(self):
+        return self._required_parameters
+
     @property
     def initial_state(self):
         """dict: Dictionary with initial states.
@@ -200,9 +222,30 @@ class UnitBaseClass(metaclass=StructMeta):
             if value is not None:
                 setattr(self, st, value)
 
+    @property
+    def missing_parameters(self):
+        missing_parameters = []
+        for param in self._required_parameters:
+            if getattr(self, param) is None:
+                missing_parameters.append(param)
+
+        missing_parameters += [
+            f'binding_model.{param}' for param in self.binding_model.missing_parameters
+        ]
+
+        return missing_parameters
+
+    def check_required_parameters(self):
+        if len(self.missing_parameters) == 0:
+            return True
+        else:
+            for param in self.missing_parameters:
+                warn(f'Unit {self.name}: Missing parameter "{param}".')
+            return False
+
     @property
     def binding_model(self):
-        """binding_model: BindingModel of the unit_operation.
+        """binding_model: BindingModel of the unit operation.
 
         Raises
         ------
@@ -226,8 +269,8 @@ class UnitBaseClass(metaclass=StructMeta):
         self._binding_model = binding_model
 
     @property
-    def _n_bound_states(self):
-        return self.binding_model.n_states
+    def n_bound_states(self):
+        return self.binding_model.n_bound_states
 
     @property
     def bulk_reaction_model(self):
@@ -308,12 +351,12 @@ class SourceMixin(metaclass=StructMeta):
     See Also
     --------
     SinkMixin
-    Source
+    Inlet
     Cstr
 
     """
     _n_poly_coeffs = 4
-    flow_rate = Polynomial(dep=('_n_poly_coeffs'), default=0)
+    flow_rate = Polynomial(dep=('_n_poly_coeffs'))
     _parameter_names = ['flow_rate']
     _section_dependent_parameters = ['flow_rate']
     _polynomial_parameters = ['flow_rate']
@@ -331,9 +374,22 @@ class SinkMixin():
     pass
 
 
-class Source(UnitBaseClass, SourceMixin):
-    """Pseudo unit operation model for streams entering the system."""
+class Inlet(UnitBaseClass, SourceMixin):
+    """Pseudo unit operation model for streams entering the system.
+
+    Attributes
+    ----------
+    c : NdPolynomial
+        Polynomial coefficients for component concentration.
+    flow_rate : NdPolynomial
+        Polynomial coefficients for volumetric flow rate.
+    solution_recorder : IORecorder
+        Solution recorder for the unit operation.
+
+    """
+
     c = NdPolynomial(dep=('n_comp', '_n_poly_coeffs'), default=0)
+    flow_rate = Polynomial(dep=('_n_poly_coeffs'), default=0)
     _n_poly_coeffs = 4
     _parameter_names = \
         UnitBaseClass._parameter_names + \
@@ -347,17 +403,25 @@ class Source(UnitBaseClass, SourceMixin):
         UnitBaseClass._polynomial_parameters + \
         SourceMixin._polynomial_parameters + \
         ['c']
+    _required_parameters = ['flow_rate']
 
 
-Inlet = Source
+Source = Inlet
 
 
-class Sink(UnitBaseClass, SinkMixin):
-    """Pseudo unit operation model for streams leaving the system."""
+class Outlet(UnitBaseClass, SinkMixin):
+    """Pseudo unit operation model for streams leaving the system.
+
+    Attributes
+    ----------
+    solution_recorder : IORecorder
+        Solution recorder for the unit operation.
+    """
+
     pass
 
 
-Outlet = Sink
+Sink = Outlet
 
 
 class MixerSplitter(UnitBaseClass):
@@ -385,12 +449,19 @@ class TubularReactorBase(UnitBaseClass):
         Diameter of column.
     axial_dispersion : UnsignedFloat
         Dispersion rate of compnents in axial direction.
+    flow_direction : Switch
+        If 1: Forward flow.
+        If -1: Backwards flow.
+    discretization : DiscretizationParametersBase
+        Discretization scheme of the unit.
 
     """
-    length = UnsignedFloat(default=0)
-    diameter = UnsignedFloat(default=0)
+
+    length = UnsignedFloat()
+    diameter = UnsignedFloat()
     axial_dispersion = UnsignedFloat()
     flow_direction = Switch(valid=[-1, 1], default=1)
+    _initial_state = UnitBaseClass._initial_state + ['c']
     _parameter_names = UnitBaseClass._parameter_names + [
         'length', 'diameter',
         'axial_dispersion', 'flow_direction'
@@ -398,6 +469,7 @@ class TubularReactorBase(UnitBaseClass):
     _section_dependent_parameters = \
         UnitBaseClass._section_dependent_parameters + \
         ['axial_dispersion', 'flow_direction']
+    _required_parameters = ['length', 'axial_dispersion', 'diameter']
 
     @abstractproperty
     def total_porosity(self):
@@ -415,7 +487,8 @@ class TubularReactorBase(UnitBaseClass):
         cross_section_area_solid
 
         """
-        return math.pi/4 * self.diameter**2
+        if self.diameter is not None:
+            return math.pi/4 * self.diameter**2
 
     @cross_section_area.setter
     def cross_section_area(self, cross_section_area):
@@ -540,7 +613,7 @@ class TubularReactorBase(UnitBaseClass):
         return self.volume_interstitial / flow_rate
 
     def u0(self, flow_rate):
-        """Flow velocity of a (non adsorbint) volume element.
+        """Flow velocity of a (non adsorbing) volume element.
 
         Parameters
         ----------
@@ -561,7 +634,7 @@ class TubularReactorBase(UnitBaseClass):
         return self.length/self.t0(flow_rate)
 
     def NTP(self, flow_rate):
-        """Number of theoretical plates.
+        r"""Number of theoretical plates.
 
         Parameters
         ----------
@@ -569,7 +642,9 @@ class TubularReactorBase(UnitBaseClass):
             volumetric flow rate
 
         Calculated using the axial dispersion coefficient:
-        :math: NTP = \frac{u \cdot L_{Column}}{2 \cdot D_a}
+
+        .. math::
+            NTP = \frac{u \cdot L_{Column}}{2 \cdot D_a}
 
         Returns
         -------
@@ -580,7 +655,8 @@ class TubularReactorBase(UnitBaseClass):
         return self.u0(flow_rate) * self.length / (2 * self.axial_dispersion)
 
     def set_axial_dispersion_from_NTP(self, NTP, flow_rate):
-        """
+        r"""Set axial dispersion from number of theoretical plates (NTP).
+
         Parameters
         ----------
         NTP : float
@@ -589,7 +665,9 @@ class TubularReactorBase(UnitBaseClass):
             volumetric flow rate
 
         Calculated using the axial dispersion coefficient:
-        :math: NTP = \frac{u \cdot L_{Column}}{2 \cdot D_a}
+
+        .. math::
+            NTP = \frac{u \cdot L_{Column}}{2 \cdot D_a}
 
         Returns
         -------
@@ -612,8 +690,10 @@ class TubularReactor(TubularReactorBase):
 
     Attributes
     ----------
-    c : List of unsinged floats. Length depends on n_comp
+    c : List of unsigned floats. Length depends on n_comp
         Initial concentration of the reactor.
+    solution_recorder : TubularReactorRecorder
+        Solution recorder for the unit operation.
 
     """
     supports_bulk_reaction = True
@@ -623,6 +703,7 @@ class TubularReactor(TubularReactorBase):
 
     c = DependentlySizedList(dep='n_comp', default=0)
     _initial_state = UnitBaseClass._initial_state + ['c']
+    _parameter_names = TubularReactorBase._parameter_names + _initial_state
 
     def __init__(self, *args, discretization_scheme='FV', **kwargs):
         super().__init__(*args, **kwargs)
@@ -642,10 +723,12 @@ class LumpedRateModelWithoutPores(TubularReactorBase):
     ----------
     total_porosity : UnsignedFloat between 0 and 1.
         Total porosity of the column.
-    c : List of unsinged floats. Length depends on n_comp
+    c : List of unsigned floats. Length depends on n_comp
             Initial concentration of the reactor.
-    q : List of unsinged floats. Length depends on n_comp
+    q : List of unsigned floats. Length depends on n_comp
         Initial concentration of the bound phase.
+    solution_recorder : LRMRecorder
+        Solution recorder for the unit operation.
 
     Notes
     -----
@@ -662,12 +745,12 @@ class LumpedRateModelWithoutPores(TubularReactorBase):
     _parameter_names = TubularReactorBase._parameter_names + [
         'total_porosity'
     ]
+    _required_parameters = TubularReactorBase._required_parameters + ['total_porosity']
 
     c = DependentlySizedList(dep='n_comp', default=0)
-    q = DependentlySizedUnsignedList(
-        dep=('n_comp', '_n_bound_states'), default=0
-    )
-    _initial_state = TubularReactorBase._initial_state + ['c', 'q']
+    _q = DependentlySizedUnsignedList(dep='n_bound_states', default=0)
+    _initial_state = TubularReactorBase._initial_state + ['q']
+    _parameter_names = _parameter_names + _initial_state
 
     def __init__(self, *args, discretization_scheme='FV', **kwargs):
         super().__init__(*args, **kwargs)
@@ -679,6 +762,18 @@ class LumpedRateModelWithoutPores(TubularReactorBase):
 
         self.solution_recorder = LRMRecorder()
 
+    @property
+    def q(self):
+        return self._q
+
+    @q.setter
+    def q(self, q):
+        if isinstance(self.binding_model, NoBinding):
+            raise CADETProcessError("Cannot set q without binding model.")
+        self._q = q
+
+        self._parameters['q'] = q
+
 
 class LumpedRateModelWithPores(TubularReactorBase):
     """Parameters for the lumped rate model with pores.
@@ -691,21 +786,23 @@ class LumpedRateModelWithPores(TubularReactorBase):
         Porosity of particles.
     particle_radius : UnsignedFloat
         Radius of the particles.
-    film_diffusion : List of unsinged floats. Length depends on n_comp.
+    film_diffusion : List of unsigned floats. Length depends on n_comp.
         Diffusion rate for components in pore volume.
-    pore_accessibility : List of unsinged floats. Length depends on n_comp.
+    pore_accessibility : List of unsigned floats. Length depends on n_comp.
         Accessibility of pores for components.
-    c : List of unsinged floats. Length depends on n_comp
+    c : List of unsigned floats. Length depends on n_comp
         Initial concentration of the reactor.
-    cp : List of unsinged floats. Length depends on n_comp
+    cp : List of unsigned floats. Length depends on n_comp
         Initial concentration of the pores
-    q : List of unsinged floats. Length depends on n_comp
+    q : List of unsigned floats. Length depends on n_comp
         Initial concntration of the bound phase.
+    solution_recorder : LRMPRecorder
+        Solution recorder for the unit operation.
 
     """
     supports_bulk_reaction = True
     supports_particle_reaction = True
-    discretization_schemes = (LRMPDiscretizationFV)
+    discretization_schemes = (LRMPDiscretizationFV, LRMPDiscretizationDG)
 
     bed_porosity = UnsignedFloat(ub=1)
     particle_porosity = UnsignedFloat(ub=1)
@@ -718,18 +815,24 @@ class LumpedRateModelWithPores(TubularReactorBase):
             ]
     _section_dependent_parameters = \
         TubularReactorBase._section_dependent_parameters + ['film_diffusion']
+    _required_parameters = TubularReactorBase._required_parameters + [
+        'bed_porosity', 'particle_porosity', 'particle_radius', 'film_diffusion'
+    ]
 
     c = DependentlySizedList(dep='n_comp', default=0)
     _cp = DependentlySizedUnsignedList(dep='n_comp')
-    q = DependentlySizedUnsignedList(
-        dep=('n_comp', '_n_bound_states'), default=0
-    )
-    _initial_state = TubularReactorBase._initial_state + ['cp', 'q']
+    _q = DependentlySizedUnsignedList(dep='n_bound_states', default=0)
 
-    def __init__(self, *args, **kwargs):
+    _initial_state = TubularReactorBase._initial_state + ['cp', 'q']
+    _parameter_names = _parameter_names + _initial_state
+
+    def __init__(self, *args, discretization_scheme='FV', **kwargs):
         super().__init__(*args, **kwargs)
 
-        self.discretization = LRMPDiscretizationFV()
+        if discretization_scheme == 'FV':
+            self.discretization = LRMPDiscretizationFV()
+        elif discretization_scheme == 'DG':
+            self.discretization = LRMPDiscretizationDG()
 
         self.solution_recorder = LRMPRecorder()
 
@@ -779,11 +882,27 @@ class LumpedRateModelWithPores(TubularReactorBase):
     def cp(self):
         if self._cp is None:
             return self.c
+        else:
+            return self._cp
 
     @cp.setter
     def cp(self, cp):
         self._cp = cp
 
+        self._parameters['cp'] = cp
+
+    @property
+    def q(self):
+        return self._q
+
+    @q.setter
+    def q(self, q):
+        if isinstance(self.binding_model, NoBinding):
+            raise CADETProcessError("Cannot set q without binding model.")
+        self._q = q
+
+        self._parameters['q'] = q
+
 
 class GeneralRateModel(TubularReactorBase):
     """Parameters for the general rate model.
@@ -796,34 +915,34 @@ class GeneralRateModel(TubularReactorBase):
         Porosity of particles.
     particle_radius : UnsignedFloat
         Radius of the particles.
-    pore_diffusion : List of unsinged floats. Length depends on n_comp.
+    pore_diffusion : List of unsigned floats. Length depends on n_comp.
         Diffusion rate for components in pore volume.
-    surface_diffusion : List of unsinged floats. Length depends on n_comp.
+    surface_diffusion : List of unsigned floats. Length depends on n_comp.
         Diffusion rate for components in adsrobed state.
-    pore_accessibility : List of unsinged floats. Length depends on n_comp.
+    pore_accessibility : List of unsigned floats. Length depends on n_comp.
         Accessibility of pores for components.
-    c : List of unsinged floats. Length depends on n_comp
+    c : List of unsigned floats. Length depends on n_comp
         Initial concentration of the reactor.
-    cp : List of unsinged floats. Length depends on n_comp
+    cp : List of unsigned floats. Length depends on n_comp
         Initial concentration of the pores
-    q : List of unsinged floats. Length depends on n_comp
+    q : List of unsigned floats. Length depends on n_comp
         Initial concntration of the bound phase.
+    solution_recorder : GRMRecorder
+        Solution recorder for the unit operation.
 
     """
     supports_bulk_reaction = True
     supports_particle_reaction = True
-    discretization_schemes = (GRMDiscretizationFV)
+    discretization_schemes = (GRMDiscretizationFV, GRMDiscretizationDG)
 
     bed_porosity = UnsignedFloat(ub=1)
     particle_porosity = UnsignedFloat(ub=1)
     particle_radius = UnsignedFloat()
     film_diffusion = DependentlySizedUnsignedList(dep='n_comp')
     pore_diffusion = DependentlySizedUnsignedList(dep='n_comp')
-    surface_diffusion = DependentlySizedUnsignedList(
-        dep=('n_comp', '_n_bound_states')
-    )
+    _surface_diffusion = DependentlySizedUnsignedList(dep='n_bound_states')
     pore_accessibility = DependentlySizedUnsignedList(dep='n_comp')
-    _parameters = \
+    _parameter_names = \
         TubularReactorBase._parameter_names + \
         [
             'bed_porosity', 'particle_porosity', 'particle_radius',
@@ -832,18 +951,25 @@ class GeneralRateModel(TubularReactorBase):
     _section_dependent_parameters = \
         TubularReactorBase._section_dependent_parameters + \
         ['film_diffusion', 'pore_diffusion', 'surface_diffusion']
+    _required_parameters = TubularReactorBase._required_parameters + [
+        'bed_porosity', 'particle_porosity', 'particle_radius', 'film_diffusion',
+        'pore_diffusion'
+    ]
 
     c = DependentlySizedList(dep='n_comp', default=0)
     _cp = DependentlySizedUnsignedList(dep='n_comp')
-    q = DependentlySizedUnsignedList(
-        dep=('n_comp', '_n_bound_states'), default=0
-    )
-    _initial_state = TubularReactorBase._initial_state + ['cp', 'q']
+    _q = DependentlySizedUnsignedList(dep='n_bound_states', default=0)
 
-    def __init__(self, *args, **kwargs):
+    _initial_state = TubularReactorBase._initial_state + ['cp', 'q']
+    _parameter_names = _parameter_names + _initial_state
+
+    def __init__(self, *args, discretization_scheme='FV', **kwargs):
         super().__init__(*args, **kwargs)
 
-        self.discretization = GRMDiscretizationFV()
+        if discretization_scheme == 'FV':
+            self.discretization = GRMDiscretizationFV()
+        elif discretization_scheme == 'DG':
+            self.discretization = GRMDiscretizationDG()
 
         self.solution_recorder = GRMRecorder()
 
@@ -893,27 +1019,59 @@ class GeneralRateModel(TubularReactorBase):
     def cp(self):
         if self._cp is None:
             return self.c
+        else:
+            return self._cp
 
     @cp.setter
     def cp(self, cp):
         self._cp = cp
 
+        self._parameters['cp'] = cp
+
+    @property
+    def q(self):
+        return self._q
+
+    @q.setter
+    def q(self, q):
+        if isinstance(self.binding_model, NoBinding):
+            raise CADETProcessError("Cannot set q without binding model.")
+        self._q = q
+
+        self._parameters['q'] = q
+
+    @property
+    def surface_diffusion(self):
+        return self._surface_diffusion
+
+    @surface_diffusion.setter
+    def surface_diffusion(self, surface_diffusion):
+        if isinstance(self.binding_model, NoBinding):
+            raise CADETProcessError(
+                "Cannot set surface diffusion without binding model."
+            )
+        self._surface_diffusion = surface_diffusion
+
+        self._parameters['_surface_diffusion'] = surface_diffusion
+
 
 class Cstr(UnitBaseClass, SourceMixin, SinkMixin):
     """Parameters for an ideal mixer.
 
     Parameters
     ----------
-    c : List of unsinged floats. Length depends on n_comp
+    c : List of unsigned floats. Length depends on n_comp
         Initial concentration of the reactor.
-    q : List of unsinged floats. Length depends on n_comp
+    q : List of unsigned floats. Length depends on n_comp
         Initial concentration of the bound phase.
-    V : Unsinged float
+    V : unsigned float
         Initial volume of the reactor.
     total_porosity : UnsignedFloat between 0 and 1.
         Total porosity of the column.
     flow_rate_filter: float:
         Flow rate of pure liquid without components to reduce volume.
+    solution_recorder : CSTRRecorder
+        Solution recorder for the unit operation.
 
     """
     supports_bulk_reaction = True
@@ -933,21 +1091,25 @@ class Cstr(UnitBaseClass, SourceMixin, SinkMixin):
         UnitBaseClass._polynomial_parameters + \
         SourceMixin._polynomial_parameters + \
         ['flow_rate_filter']
+    _required_parameters = UnitBaseClass._required_parameters + ['V']
 
     c = DependentlySizedList(dep='n_comp', default=0)
-    q = DependentlySizedUnsignedList(
-        dep=('n_comp', '_n_bound_states'), default=0
-    )
-    V = UnsignedFloat(default=0)
-    volume = V
+    _q = DependentlySizedUnsignedList(dep='n_bound_states', default=0)
+    V = UnsignedFloat()
     _initial_state = \
         UnitBaseClass._initial_state + \
         ['c', 'q', 'V']
+    _parameter_names = _parameter_names + _initial_state
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.solution_recorder = CSTRRecorder()
 
+    @property
+    def volume(self):
+        """float: Alias for volume."""
+        return self.V
+
     @property
     def volume_liquid(self):
         """float: Volume of the liquid phase."""
@@ -977,3 +1139,15 @@ class Cstr(UnitBaseClass, SourceMixin, SinkMixin):
 
         """
         return self.volume_liquid / flow_rate
+
+    @property
+    def q(self):
+        return self._q
+
+    @q.setter
+    def q(self, q):
+        if isinstance(self.binding_model, NoBinding):
+            raise CADETProcessError("Cannot set q without binding model.")
+        self._q = q
+
+        self._parameters['q'] = q
