@@ -1,3 +1,4 @@
+from collections import defaultdict
 import os
 import platform
 from pathlib import Path
@@ -12,6 +13,7 @@ import numpy as np
 from cadet import Cadet as CadetAPI
 
 from CADETProcess import CADETProcessError
+from CADETProcess import settings
 from CADETProcess.dataStructure import (
     Bool, Switch, UnsignedFloat, UnsignedInteger,
 )
@@ -25,11 +27,24 @@ from CADETProcess.processModel import NoBinding, BindingBaseClass
 from CADETProcess.processModel import NoReaction, ReactionBaseClass
 from CADETProcess.processModel import NoDiscretization, DGMixin
 from CADETProcess.processModel import (
-    UnitBaseClass, Source, Cstr, LumpedRateModelWithoutPores
+    UnitBaseClass, Inlet, Cstr, TubularReactor, LumpedRateModelWithoutPores
 )
 from CADETProcess.processModel import Process
 
 
+__all__ = [
+    'Cadet',
+    'ModelSolverParametersGroup',
+    'UnitParametersGroup',
+    'AdsorptionParametersGroup',
+    'ReactionParametersGroup',
+    'SolverParametersGroup',
+    'SolverTimeIntegratorParametersGroup',
+    'ReturnParametersGroup',
+    'SensitivityParametersGroup',
+]
+
+
 class Cadet(SimulatorBase):
     """CADET class for running a simulation for given process objects.
 
@@ -37,8 +52,6 @@ class Cadet(SimulatorBase):
     ----------
     install_path: str
         Path to the installation of CADET
-    temp_dir : str
-        Path to directory for temporary files
     time_out : UnsignedFloat
         Maximum duration for simulations
     model_solver_parameters : ModelSolverParametersGroup
@@ -56,10 +69,9 @@ class Cadet(SimulatorBase):
     return_parameters : ReturnParametersGroup
         Container for return information of the system
 
-    Todo
-    ----
-    Implement method for loading CADET file that have not been generated
-    with CADETProcess and create Process
+    ..todo::
+        Implement method for loading CADET file that have not been generated
+        with CADETProcess and create Process
 
     See Also
     --------
@@ -70,6 +82,7 @@ class Cadet(SimulatorBase):
     CadetAPI
 
     """
+
     timeout = UnsignedFloat()
     use_c_api = Bool(default=False)
     _force_constant_flow_rate = False
@@ -78,17 +91,31 @@ class Cadet(SimulatorBase):
         super().__init__(*args, **kwargs)
 
         self.install_path = install_path
-        self.temp_dir = temp_dir
 
         self.model_solver_parameters = ModelSolverParametersGroup()
         self.solver_parameters = SolverParametersGroup()
         self.time_integrator_parameters = SolverTimeIntegratorParametersGroup()
 
         self.return_parameters = ReturnParametersGroup()
+        self.sensitivity_parameters = SensitivityParametersGroup()
+
+        if temp_dir is None:
+            temp_dir = settings.temp_dir / 'simulation_files'
+        self.temp_dir = temp_dir
+
+    @property
+    def temp_dir(self):
+        if not self._temp_dir.exists():
+            self._temp_dir.mkdir(exist_ok=True, parents=True)
+        return self._temp_dir
+
+    @temp_dir.setter
+    def temp_dir(self, temp_dir):
+        self._temp_dir = temp_dir
 
     @property
     def install_path(self):
-        """str: Path to the installation of CADET
+        """str: Path to the installation of CADET.
 
         Parameters
         ----------
@@ -103,7 +130,7 @@ class Cadet(SimulatorBase):
 
         See Also
         --------
-        check_cadet()
+        check_cadet
 
         """
         return self._install_path
@@ -126,7 +153,7 @@ class Cadet(SimulatorBase):
                 raise FileNotFoundError(
                     "CADET could not be found. Please set an install path"
                 )
-        install_path = Path(install_path)
+        install_path = Path(install_path).expanduser()
 
         if install_path.exists():
             self._install_path = install_path
@@ -148,7 +175,7 @@ class Cadet(SimulatorBase):
             os.environ['LD_LIBRARY_PATH'] = cadet_lib_path.as_posix()
 
     def check_cadet(self):
-        """Wrapper around a basic CADET example for testing functionality"""
+        """Check CADET installation can run basic LWE example."""
         executable = 'createLWE'
         if platform.system() == 'Windows':
             executable += '.exe'
@@ -185,28 +212,12 @@ class Cadet(SimulatorBase):
                 "Simulation failed"
             )
 
-    @property
-    def temp_dir(self):
-        return tempfile.gettempdir()
-
-    @temp_dir.setter
-    def temp_dir(self, temp_dir):
-        if temp_dir is not None:
-            try:
-                exists = Path(temp_dir).exists()
-            except TypeError:
-                raise CADETProcessError('Not a valid path')
-            if not exists:
-                raise CADETProcessError('Not a valid path')
-
-        tempfile.tempdir = temp_dir
-
     def get_tempfile_name(self):
         f = next(tempfile._get_candidate_names())
-        return os.path.join(self.temp_dir, f + '.h5')
+        return self.temp_dir / f'{f}.h5'
 
     def run(self, process, cadet=None, file_path=None):
-        """Interface to the solver run function
+        """Interface to the solver run function.
 
         The configuration is extracted from the process object and then saved
         as a temporary .h5 file. After termination, the same file is processed
@@ -259,7 +270,10 @@ class Cadet(SimulatorBase):
             return_information = cadet.run_load(timeout=self.timeout)
             elapsed = time.time() - start
         except TimeoutExpired:
-            raise CADETProcessError('Simulator timed out')
+            raise CADETProcessError('Simulator timed out') from None
+        finally:
+            if file_path is None:
+                os.remove(cadet.filename)
 
         if return_information.returncode != 0:
             self.logger.error(
@@ -267,8 +281,8 @@ class Cadet(SimulatorBase):
                 f'with parameters {process.config} failed.'
             )
             raise CADETProcessError(
-                f'CADET Error: {return_information.stderr}'
-            )
+                f'CADET Error: Simulation failed with {return_information.stderr}'
+            ) from None
 
         try:
             results = self.get_simulation_results(
@@ -279,10 +293,6 @@ class Cadet(SimulatorBase):
                 'Unexpected error reading SimulationResults.'
             )
 
-        # Remove files
-        if file_path is None:
-            os.remove(cadet.filename)
-
         return results
 
     def save_to_h5(self, process, file_path):
@@ -320,16 +330,20 @@ class Cadet(SimulatorBase):
 
         See Also
         --------
-        input_model
-        input_solver
-        input_return
+        get_input_model
+        get_input_solver
+        get_input_return
+        get_input_sensitivity
 
         """
         process.lock = True
+
         config = Dict()
         config.input.model = self.get_input_model(process)
         config.input.solver = self.get_input_solver(process)
         config.input['return'] = self.get_input_return(process)
+        config.input.sensitivity = self.get_input_sensitivity(process)
+
         process.lock = False
 
         return config
@@ -364,9 +378,8 @@ class Cadet(SimulatorBase):
         results : SimulationResults
             Simulation results including process and solver configuration.
 
-        Todo
-        ----
-        Implement method to read .h5 files that have no process associated.
+        ..todo::
+            Implement method to read .h5 files that have no process associated.
 
         """
         if time_elapsed is None:
@@ -379,9 +392,8 @@ class Cadet(SimulatorBase):
             exit_flag = return_information.returncode
             exit_message = return_information.stderr.decode()
 
-        solution = Dict()
-        from collections import defaultdict
         try:
+            solution = Dict()
             for unit in process.flow_sheet.units:
                 solution[unit.name] = defaultdict(list)
                 unit_index = self.get_unit_index(process, unit)
@@ -395,8 +407,8 @@ class Cadet(SimulatorBase):
                 flow_out = process.flow_rate_timelines[unit.name].total_out
 
                 for cycle in range(self.n_cycles):
-                    start = cycle * (len(time) - 1)
-                    end = (cycle + 1) * (len(time) - 1) + 1
+                    start = cycle * len(time)
+                    end = (cycle + 1) * len(time)
 
                     if 'solution_inlet' in unit_solution.keys():
                         sol_inlet = unit_solution.solution_inlet[start:end, :]
@@ -409,8 +421,7 @@ class Cadet(SimulatorBase):
                         )
 
                     if 'solution_outlet' in unit_solution.keys():
-                        sol_outlet = \
-                            unit_solution.solution_outlet[start:end, :]
+                        sol_outlet = unit_solution.solution_outlet[start:end, :]
                         solution[unit.name]['outlet'].append(
                             SolutionIO(
                                 unit.name,
@@ -430,8 +441,7 @@ class Cadet(SimulatorBase):
                         )
 
                     if 'solution_particle' in unit_solution.keys():
-                        sol_particle = \
-                            unit_solution.solution_particle[start:end, :]
+                        sol_particle = unit_solution.solution_particle[start:end, :]
                         solution[unit.name]['particle'].append(
                             SolutionParticle(
                                 unit.name,
@@ -447,7 +457,7 @@ class Cadet(SimulatorBase):
                             SolutionSolid(
                                 unit.name,
                                 unit.component_system,
-                                unit.binding_model.n_states,
+                                unit.binding_model.bound_states,
                                 time, sol_solid,
                                 **unit_coordinates,
                                 particle_coordinates=particle_coordinates
@@ -455,8 +465,7 @@ class Cadet(SimulatorBase):
                         )
 
                     if 'solution_volume' in unit_solution.keys():
-                        sol_volume = \
-                            unit_solution.solution_volume[start:end, :]
+                        sol_volume = unit_solution.solution_volume[start:end, :]
                         solution[unit.name]['volume'].append(
                             SolutionVolume(
                                 unit.name,
@@ -468,6 +477,92 @@ class Cadet(SimulatorBase):
 
             solution = Dict(solution)
 
+            sensitivity = Dict()
+            for i, sens in enumerate(process.parameter_sensitivities):
+                sens_index = f'param_{i:03d}'
+                for unit in process.flow_sheet.units:
+                    sensitivity[sens.name][unit.name] = defaultdict(list)
+                    unit_index = self.get_unit_index(process, unit)
+                    unit_sensitivity = cadet.root.output.sensitivity[sens_index][unit_index]
+                    unit_coordinates = \
+                        cadet.root.output.coordinates[unit_index].copy()
+                    particle_coordinates = \
+                        unit_coordinates.pop('particle_coordinates_000', None)
+
+                    flow_in = process.flow_rate_timelines[unit.name].total_in
+                    flow_out = process.flow_rate_timelines[unit.name].total_out
+
+                    for cycle in range(self.n_cycles):
+                        start = cycle * len(time)
+                        end = (cycle + 1) * len(time)
+
+                        if 'sens_inlet' in unit_sensitivity.keys():
+                            sens_inlet = unit_sensitivity.sens_inlet[start:end, :]
+                            sensitivity[sens.name][unit.name]['inlet'].append(
+                                SolutionIO(
+                                    unit.name,
+                                    unit.component_system, time, sens_inlet,
+                                    flow_in
+                                )
+                            )
+
+                        if 'sens_outlet' in unit_sensitivity.keys():
+                            sens_outlet = unit_sensitivity.sens_outlet[start:end, :]
+                            sensitivity[sens.name][unit.name]['outlet'].append(
+                                SolutionIO(
+                                    unit.name,
+                                    unit.component_system, time, sens_outlet,
+                                    flow_out
+                                )
+                            )
+
+                        if 'sens_bulk' in unit_sensitivity.keys():
+                            sens_bulk = unit_sensitivity.sens_bulk[start:end, :]
+                            sensitivity[sens.name][unit.name]['bulk'].append(
+                                SolutionBulk(
+                                    unit.name,
+                                    unit.component_system, time, sens_bulk,
+                                    **unit_coordinates
+                                )
+                            )
+
+                        if 'sens_particle' in unit_sensitivity.keys():
+                            sens_particle = unit_sensitivity.sens_particle[start:end, :]
+                            sensitivity[sens.name][unit.name]['particle'].append(
+                                SolutionParticle(
+                                    unit.name,
+                                    unit.component_system, time, sens_particle,
+                                    **unit_coordinates,
+                                    particle_coordinates=particle_coordinates
+                                )
+                            )
+
+                        if 'sens_solid' in unit_sensitivity.keys():
+                            sens_solid = unit_sensitivity.sens_solid[start:end, :]
+                            sensitivity[sens.name][unit.name]['solid'].append(
+                                SolutionSolid(
+                                    unit.name,
+                                    unit.component_system,
+                                    unit.binding_model.bound_states,
+                                    time, sens_solid,
+                                    **unit_coordinates,
+                                    particle_coordinates=particle_coordinates
+                                )
+                            )
+
+                        if 'sens_volume' in unit_sensitivity.keys():
+                            sens_volume = unit_sensitivity.sens_volume[start:end, :]
+                            sensitivity[sens.name][unit.name]['volume'].append(
+                                SolutionVolume(
+                                    unit.name,
+                                    unit.component_system,
+                                    time,
+                                    sens_volume
+                                )
+                            )
+
+            sensitivity = Dict(sensitivity)
+
             system_state = {
                 'state': cadet.root.output.last_state_y,
                 'state_derivative': cadet.root.output.last_state_ydot
@@ -475,7 +570,7 @@ class Cadet(SimulatorBase):
 
             chromatograms = [
                 solution[chrom.name].outlet[-1]
-                for chrom in process.flow_sheet.chromatogram_sinks
+                for chrom in process.flow_sheet.product_outlets
             ]
 
         except KeyError:
@@ -489,6 +584,7 @@ class Cadet(SimulatorBase):
             time_elapsed=time_elapsed,
             process=process,
             solution_cycles=solution,
+            sensitivity_cycles=sensitivity,
             system_state=system_state,
             chromatograms=chromatograms
         )
@@ -656,7 +752,12 @@ class Cadet(SimulatorBase):
         unit_config = Dict(unit_parameters.to_dict())
 
         if not isinstance(unit.binding_model, NoBinding):
-            n_bound = [unit.binding_model.n_states] * unit.binding_model.n_comp
+            if unit.binding_model.n_binding_sites > 1:
+                n_bound = \
+                    [unit.binding_model.n_binding_sites] * unit.binding_model.n_comp
+            else:
+                n_bound = unit.binding_model.bound_states
+
             unit_config['adsorption'] = \
                 self.get_adsorption_config(unit.binding_model)
             unit_config['adsorption_model'] = \
@@ -677,8 +778,15 @@ class Cadet(SimulatorBase):
 
         if not isinstance(unit.bulk_reaction_model, NoReaction):
             parameters = self.get_reaction_config(unit.bulk_reaction_model)
-            unit_config['reaction_model'] = parameters['REACTION_MODEL']
-            unit_config['reaction_bulk'] = parameters
+
+            if isinstance(unit, TubularReactor):
+                unit_config['reaction_model'] = parameters['REACTION_MODEL']
+                for key, value in parameters.items():
+                    key = key.replace('bulk', 'liquid')
+                    unit_config['reaction'][key] = value
+            else:
+                unit_config['reaction_model'] = parameters['REACTION_MODEL']
+                unit_config['reaction_bulk'] = parameters
 
         if not isinstance(unit.particle_reaction_model, NoReaction):
             parameters = self.get_reaction_config(unit.particle_reaction_model)
@@ -689,7 +797,7 @@ class Cadet(SimulatorBase):
                 unit_config['reaction_model_particle'] = parameters['REACTION_MODEL']
                 unit_config['reaction_particle'].update(parameters)
 
-        if isinstance(unit, Source):
+        if isinstance(unit, Inlet):
             unit_config['sec_000']['const_coeff'] = unit.c[:, 0]
             unit_config['sec_000']['lin_coeff'] = unit.c[:, 1]
             unit_config['sec_000']['quad_coeff'] = unit.c[:, 2]
@@ -720,7 +828,7 @@ class Cadet(SimulatorBase):
                     if param_name == 'flow_rate':
                         continue
                     unit_index = process.flow_sheet.get_unit_index(unit)
-                    if isinstance(unit, Source) and param_name == 'c':
+                    if isinstance(unit, Inlet) and param_name == 'c':
                         self.add_inlet_section(
                             model_units, section_index, unit_index, state
                         )
@@ -840,6 +948,95 @@ class Cadet(SimulatorBase):
 
         return unit_return_parameters
 
+    def get_input_sensitivity(self, process):
+        """Config branch /input/sensitivity"""
+        sensitivity_parameters = self.sensitivity_parameters.to_dict()
+        parameter_sensitivities = self.get_parameter_sensitivities(process)
+        return {**sensitivity_parameters, **parameter_sensitivities}
+
+    def get_parameter_sensitivities(self, process):
+        """Config branches for all parameter sensitivities /input/sensitivity/param_000 ... param_xxx"""
+        parameter_sensitivities = Dict()
+        parameter_sensitivities.nsens = process.n_sensitivities
+        for i, sens in enumerate(process.parameter_sensitivities):
+            sens_index = f'param_{i:03d}'
+            parameter_sensitivities[sens_index] = \
+                self.get_sensitivity_config(process, sens)
+
+        return parameter_sensitivities
+
+    def get_sensitivity_config(self, process, sens):
+        config = Dict()
+
+        unit_indices = []
+        parameters = []
+        components = []
+
+        for param, unit, associated_model, comp, coeff in zip(
+                sens.parameters, sens.units, sens.associated_models, sens.components,
+                sens.polynomial_coefficients):
+            unit_index = process.flow_sheet.get_unit_index(unit)
+            unit_indices.append(unit_index)
+
+            if associated_model is None:
+                model = unit.model
+                if model == 'Inlet' and param == 'c':
+                    if coeff == 0:
+                        coeff = 'CONST_COEFF'
+                    elif coeff == 1:
+                        coeff = 'CONST_COEFF'
+                    elif coeff == 2:
+                        coeff = 'QUAD_COEFF'
+                    elif coeff == 3:
+                        coeff = 'CUBE_COEFF'
+                    parameter = coeff
+                else:
+                    parameter = inv_unit_parameters_map[model]['parameters'][param]
+            else:
+                model = associated_model.model
+                if isinstance(associated_model, BindingBaseClass):
+                    parameter = inv_adsorption_parameters_map[model]['parameters'][param]
+                if isinstance(associated_model, ReactionBaseClass):
+                    parameter = inv_reaction_parameters_map[model]['parameters'][param]
+            parameters.append(parameter)
+
+            component_system = unit.component_system
+            comp = -1 if comp is None else component_system.indices[comp]
+            components.append(comp)
+
+        config.sens_unit = unit_indices
+        config.sens_name = parameters
+        config.sens_comp = components
+
+        config.sens_partype = -1    # !!! Check when multiple particle types enabled.
+        if not all([index is None for index in sens.bound_state_indices]):
+            config.sens_reaction = [
+                -1 if index is None else index for index in sens.bound_state_indices
+            ]
+        else:
+            config.sens_reaction = -1
+
+        if not all([index is None for index in sens.bound_state_indices]):
+            config.sens_boundphase = [
+                -1 if index is None else index for index in sens.bound_state_indices
+            ]
+        else:
+            config.sens_boundphase = -1
+
+        if not all([index is None for index in sens.section_indices]):
+            config.sens_section = [
+                -1 if index is None else index for index in sens.section_indices
+            ]
+        else:
+            config.sens_section = -1
+
+        if not all([index is None for index in sens.abstols]):
+            config.sens_abstol = sens.abstols
+
+        config.factors = sens.factors
+
+        return config
+
     def __str__(self):
         return 'CADET'
 
@@ -848,22 +1045,47 @@ from CADETProcess.dataStructure import ParametersGroup, ParameterWrapper
 class ModelSolverParametersGroup(ParametersGroup):
     """Converter for model solver parameters from CADETProcess to CADET.
 
+    Attributes
+    ----------
+    gs_type : {1, 0}, optional
+        Valid modes:
+        - 0: Classical Gram-Schmidet orthogonalization.
+        - 1: Modified Gram-Schmidt.
+        The default is 1.
+    max_krylov : int, optional
+        Size of the Krylov subspace in the iterative linear GMRES solver.
+        The default is 0.
+    max_restarts : int, optional
+        Maximum number of restarts in the GMRES algorithm. If lack of memory is not an
+        issue, better use a larger Krylov space than restarts.
+        The default is 10.
+    schur_safety : float, optional
+        Schur safety factor.
+        Influences the tradeoff between linear iterations and nonlinear error control
+        The default is 1e-8.
+    linear_solution_mode : int
+        Valid modes:
+        - 0: Automatically chose mode based on heuristic.
+        - 1: Solve system of models in parallel
+        - 2: Solve system of models sequentially (only possible for systems without cyclic connections)
+        The default is 0.
+
     See Also
     --------
     ParametersGroup
 
     """
-    GS_TYPE = UnsignedInteger(default=1, ub=1)
-    MAX_KRYLOV = UnsignedInteger(default=0)
-    MAX_RESTARTS = UnsignedInteger(default=10)
-    SCHUR_SAFETY = UnsignedFloat(default=1e-8)
-    LINEAR_SOLUTION_MODE = UnsignedInteger(default=2, ub=2)
+    gs_type = Switch(default=1, valid=[0, 1])
+    max_krylov = UnsignedInteger(default=0)
+    max_restarts = UnsignedInteger(default=10)
+    schur_safety = UnsignedFloat(default=1e-8)
+    linear_solution_mode = UnsignedInteger(default=0, ub=2)
     _parameters = [
-        'GS_TYPE',
-        'MAX_KRYLOV',
-        'MAX_RESTARTS',
-        'SCHUR_SAFETY',
-        'LINEAR_SOLUTION_MODE',
+        'gs_type',
+        'max_krylov',
+        'max_restarts',
+        'schur_safety',
+        'linear_solution_mode',
     ]
 
 
@@ -947,7 +1169,7 @@ unit_parameters_map = {
             'FLOWRATE_FILTER': 'flow_rate_filter',
         },
     },
-    'Source': {
+    'Inlet': {
         'name': 'INLET',
         'parameters': {
             'NCOMP': 'n_comp',
@@ -956,7 +1178,7 @@ unit_parameters_map = {
             'INLET_TYPE': 'PIECEWISE_CUBIC_POLY',
         },
     },
-    'Sink': {
+    'Outlet': {
         'name': 'OUTLET',
         'parameters': {
             'NCOMP': 'n_comp',
@@ -1064,12 +1286,12 @@ adsorption_parameters_map = {
         'name': 'STERIC_MASS_ACTION',
         'parameters': {
             'IS_KINETIC': 'is_kinetic',
-            'SMA_KA': 'adsorption_rate',
-            'SMA_KD': 'desorption_rate',
+            'SMA_KA': 'adsorption_rate_transformed',
+            'SMA_KD': 'desorption_rate_transformed',
             'SMA_LAMBDA': 'capacity',
             'SMA_NU': 'characteristic_charge',
             'SMA_SIGMA': 'steric_factor',
-            'SMA_REF0': 'reference_liquid_phase_conc',
+            'SMA_REFC0': 'reference_liquid_phase_conc',
             'SMA_REFQ': 'reference_solid_phase_conc'
         },
     },
@@ -1105,6 +1327,33 @@ adsorption_parameters_map = {
             'EMPM_GAMMA': 'hydrophobicity',
             'EMPM_COMP_MODE': 'component_mode',
         },
+    },
+    'GeneralizedIonExchange': {
+        'name': 'GENERALIZED_ION_EXCHANGE',
+        'parameters': {
+            'IS_KINETIC': 'is_kinetic',
+            'GIEX_KA': 'adsorption_rate',
+            'GIEX_KA_LIN': 'adsorption_rate_linear',
+            'GIEX_KA_QUAD': 'adsorption_rate_quadratic',
+            'GIEX_KA_CUBE': 'adsorption_rate_cubic',
+            'GIEX_KA_SALT': 'adsorption_rate_salt',
+            'GIEX_KA_PROT': 'adsorption_rate_protein',
+            'GIEX_KD': 'desorption_rate',
+            'GIEX_KD_LIN': 'desorption_rate_linear',
+            'GIEX_KD_QUAD': 'desorption_rate_quadratic',
+            'GIEX_KD_CUBE': 'desorption_rate_cubic',
+            'GIEX_KD_SALT': 'desorption_rate_salt',
+            'GIEX_KD_PROT': 'desorption_rate_protein',
+            'GIEX_NU_BREAKS': 'characteristic_charge_breaks',
+            'GIEX_NU': 'characteristic_charge',
+            'GIEX_NU_LIN': 'characteristic_charge_linear',
+            'GIEX_NU_QUAD': 'characteristic_charge_quadratic',
+            'GIEX_NU_CUBE': 'characteristic_charge_cubic',
+            'GIEX_SIGMA': 'steric_factor',
+            'GIEX_LAMBDA': 'capacity',
+            'GIEX_REFC0': 'reference_liquid_phase_conc',
+            'GIEX_REFQ': 'reference_solid_phase_conc',
+        },
     }
 }
 
@@ -1121,7 +1370,7 @@ inv_adsorption_parameters_map = {
 class AdsorptionParametersGroup(ParameterWrapper):
     """Converter for Binding model parameters from CADETProcess to CADET.
 
-    See also
+    See Also
     --------
     ParameterWrapper
     ReactionParametersGroup
@@ -1135,70 +1384,108 @@ class AdsorptionParametersGroup(ParameterWrapper):
     _model_type = 'ADSORPTION_MODEL'
 
 
+reaction_parameters_map = {
+    'NoReaction': {
+        'name': 'NONE',
+        'parameters': {},
+    },
+    'MassActionLaw': {
+        'name': 'MASS_ACTION_LAW',
+        'parameters': {
+            'mal_stoichiometry_bulk': 'stoich',
+            'mal_exponents_bulk_fwd': 'exponents_fwd',
+            'mal_exponents_bulk_bwd': 'exponents_bwd',
+            'mal_kfwd_bulk': 'k_fwd',
+            'mal_kbwd_bulk': 'k_bwd',
+            }
+    },
+    'MassActionLawParticle': {
+        'name': 'MASS_ACTION_LAW',
+        'parameters': {
+            'mal_stoichiometry_liquid': 'stoich_liquid',
+            'mal_exponents_liquid_fwd': 'exponents_fwd_liquid',
+            'mal_exponents_liquid_bwd': 'exponents_bwd_liquid',
+            'mal_kfwd_liquid': 'k_fwd_liquid',
+            'mal_kbwd_liquid': 'k_bwd_liquid',
+
+            'mal_stoichiometry_solid': 'stoich_solid',
+            'mal_exponents_solid_fwd': 'exponents_fwd_solid',
+            'mal_exponents_solid_bwd': 'exponents_bwd_solid',
+            'mal_kfwd_solid': 'k_fwd_solid',
+            'mal_kbwd_solid': 'k_bwd_solid',
+
+            'mal_exponents_liquid_fwd_modsolid':
+                'exponents_fwd_liquid_modsolid',
+            'mal_exponents_liquid_bwd_modsolid':
+                'exponents_bwd_liquid_modsolid',
+            'mal_exponents_solid_fwd_modliquid':
+                'exponents_fwd_solid_modliquid',
+            'mal_exponents_solid_bwd_modliquid':
+                'exponents_bwd_solid_modliquid',
+        }
+    }
+}
+
+
+inv_reaction_parameters_map = {
+    model: {
+        'name': values['name'],
+        'parameters': {
+            v: k for k, v in values['parameters'].items()
+        }
+    } for model, values in adsorption_parameters_map.items()
+}
+
+
 class ReactionParametersGroup(ParameterWrapper):
     """Converter for Reaction model parameters from CADETProcess to CADET.
 
     See Also
     --------
     ParameterWrapper
-    ReactionParametersGroup
+    AdsorptionParametersGroup
     UnitParametersGroup
-
     """
     _baseClass = ReactionBaseClass
 
-    _reaction_models = {
-        'NoReaction': 'NONE',
-        'MassAction': 'MASS_ACTION_LAW',
-                }
-    _reaction_parameters = {
-        'NoReaction': {
-            'name': 'NONE',
-            'parameters': {},
-        },
-        'MassActionLaw': {
-            'name': 'MASS_ACTION_LAW',
-            'parameters': {
-                'mal_stoichiometry_bulk': 'stoich',
-                'mal_exponents_bulk_fwd': 'exponents_fwd',
-                'mal_exponents_bulk_bwd': 'exponents_bwd',
-                'mal_kfwd_bulk': 'k_fwd',
-                'mal_kbwd_bulk': 'k_bwd',
-                }
-        },
-        'MassActionLawParticle': {
-            'name': 'MASS_ACTION_LAW',
-            'parameters': {
-                'mal_stoichiometry_liquid': 'stoich_liquid',
-                'mal_exponents_liquid_fwd': 'exponents_fwd_liquid',
-                'mal_exponents_liquid_bwd': 'exponents_bwd_liquid',
-                'mal_kfwd_liquid': 'k_fwd_liquid',
-                'mal_kbwd_liquid': 'k_bwd_liquid',
-
-                'mal_stoichiometry_solid': 'stoich_solid',
-                'mal_exponents_solid_fwd': 'exponents_fwd_solid',
-                'mal_exponents_solid_bwd': 'exponents_bwd_solid',
-                'mal_kfwd_solid': 'k_fwd_solid',
-                'mal_kbwd_solid': 'k_bwd_solid',
-
-                'mal_exponents_liquid_fwd_modsolid':
-                    'exponents_fwd_liquid_modsolid',
-                'mal_exponents_liquid_bwd_modsolid':
-                    'exponents_bwd_liquid_modsolid',
-                'mal_exponents_solid_fwd_modliquid':
-                    'exponents_fwd_solid_modliquid',
-                'mal_exponents_solid_bwd_modliquid':
-                    'exponents_bwd_solid_modliquid',
-            }
-        }
-    }
+    _reaction_parameters = reaction_parameters_map
 
     _model_parameters = _reaction_parameters
     _model_type = 'REACTION_MODEL'
 
 
 class SolverParametersGroup(ParametersGroup):
-    """Class for defining the solver parameters for cadet.
+    """Class for defining the solver parameters for CADET.
+
+    Attributes
+    ----------
+    nthreads : int
+        Number of used threads.
+    consistent_init_mode : int, optional
+        Consistent initialization mode.
+        Valid values are:
+        - 0: None
+        - 1: Full
+        - 2: Once, full
+        - 3: Lean
+        - 4: Once, lean
+        - 5: Full once, then lean
+        - 6: None once, then full
+        - 7: None once, then lean
+        The default is 1.
+    consistent_init_mode_sens : int, optional
+        Consistent initialization mode for parameter sensitivities.
+        Valid values are:
+        - 0: None
+        - 1: Full
+        - 2: Once, full
+        - 3: Lean
+        - 4: Once, lean
+        - 5: Full once, then lean
+        - 6: None once, then full
+        - 7: None once, then lean
+        The default is 1.
+
 
     See Also
     --------
@@ -1217,6 +1504,45 @@ class SolverParametersGroup(ParametersGroup):
 class SolverTimeIntegratorParametersGroup(ParametersGroup):
     """Converter for time integartor parameters from CADETProcess to CADET.
 
+    Attributes
+    ----------
+    abstol: float, optional
+        Absolute tolerance in the solution of the original system.
+        The default is 1e-8.
+    algtol: float, optional
+        Tolerance in the solution of the nonlinear consistency equations.
+        The default is 1e-12.
+    reltol: float, optional
+        Relative tolerance in the solution of the original system.
+        The default is 1e-6.
+    reltol_sens: float, optional
+        Relative tolerance in the solution of the sensitivity systems.
+        The default is 1e-12.
+    init_step_size: float, optional
+        Initial time integrator step size.
+        The default is 1e-6.
+    max_steps: int, optional
+        Maximum number of timesteps taken by IDAS
+        The default is 1000000.
+    max_step_size: float, optional
+        Maximum size of timesteps taken by IDAS.
+        The default is 0.0 (unlimited).
+    errortest_sens: bool, optional
+        If True: Use (forward) sensitivities in local error test
+        The default is True.
+    max_newton_iter: int, optional
+        Maximum number of Newton iterations in time step.
+        The default is 3.
+    max_errtest_fail: int, optional
+        Maximum number of local error test failures in time step
+        The default is 7.
+    max_convtest_fail: int, optional
+        Maximum number of Newton convergence test failures
+        The default is 10.
+    max_newton_iter_sens: int, optional
+        Maximum number of Newton iterations in forward sensitivity time step
+        The default is 3.
+
     See Also
     --------
     ParametersGroup
@@ -1270,5 +1596,6 @@ class SensitivityParametersGroup(ParametersGroup):
     ParametersGroup
 
     """
-    nsens = UnsignedInteger(default=0)
     sens_method = Switch(default='ad1', valid=['ad1'])
+
+    _parameters = ['sens_method']
