@@ -1,17 +1,36 @@
+"""
+=========================================
+Smoothing (:mod:`CADETProcess.smoothing`)
+=========================================
+
+.. currentmodule:: CADETProcess.smoothing
+
+This module provides functionality for smoothing data.
+
+
+.. autosummary::
+    :toctree: generated/
+
+    find_smoothing_factors
+    full_smooth
+
+"""
+
 import multiprocessing
 
 import numpy as np
 import scipy.signal
 
 from pymoo.core.problem import ElementwiseProblem
-from pymoo.factory import get_algorithm
+from pymoo.algorithms.soo.nonconvex.pattern import PatternSearch
 from pymoo.optimize import minimize
 
 butter_order = 3
 
+__all__ = ['find_smoothing_factors', 'full_smooth']
+
 
 class TargetProblem(ElementwiseProblem):
-
     def __init__(self, lb, ub, sse_target, func, values, fs):
         super().__init__(n_var=1, n_obj=1, n_constr=0, xl=lb, xu=ub)
         self.sse_target = sse_target
@@ -27,13 +46,12 @@ class TargetProblem(ElementwiseProblem):
             sse = np.sum((low_passed - self.values) ** 2)
 
             error = (sse - self.sse_target)**2
-        except ValueError:
+        except (ValueError, np.linalg.LinAlgError):
             error = np.inf
         out["F"] = error
 
 
 class MaxDistance(ElementwiseProblem):
-
     def __init__(self, lb, ub, func, fs, values, x_min, y_min, p1, p2, factor):
         super().__init__(n_var=1, n_obj=1, n_constr=0, xl=lb, xu=ub)
         self.func = func
@@ -112,7 +130,7 @@ def refine_signal(func, times, values, x, y, fs, start):
         lb, ub, func, fs, values, x_min, y_min, p1, p2, factor
     )
 
-    algorithm = get_algorithm('pattern-search', n_sample_points=50, eps=1e-13)
+    algorithm = PatternSearch(n_sample_points=50, eps=1e-13)
 
     res = minimize(
         problem,
@@ -187,7 +205,7 @@ def find_max_signal(func, times, values, sse_target, filters, sse):
         filters[0], filters[-1], sse_target, func, values, fs
     )
 
-    algorithm = get_algorithm('pattern-search', n_sample_points=50, eps=1e-13)
+    algorithm = PatternSearch(n_sample_points=50, eps=1e-13)
 
     res = minimize(
         problem,
@@ -213,7 +231,10 @@ def smoothing_filter_signal(func, times, values, crit_fs):
 def find_smoothing_factors(times, values, rmse_target=1e-4):
     sse_target = (rmse_target**2.0)*len(values)
 
-    crit_fs = find_signal(signal_bessel, times, values, sse_target)
+    try:
+        crit_fs = find_signal(signal_bessel, times, values, sse_target)
+    except np.linalg.LinAlgError:
+        crit_fs = None
 
     if crit_fs is None:
         multiprocessing.get_logger().info(
